--------------------------------------------------------------------------------
-- CERN BE-CO-HT
--
-- https://www.ohwr.org/projects/wr2rf-vme
--------------------------------------------------------------------------------
--
-- unit name:   wr2rf_dds
--
-- description: Provides wrapper for AD9910 related code
--
--------------------------------------------------------------------------------
-- Copyright CERN 2020
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

use work.wishbone_pkg.all;

entity wr2rf_dds is
   generic (
      g_use_fine_delay  : boolean := false );    
   port (
      clk_sys_i        : in  std_logic;
      clk_ref_i        : in  std_logic;        
      rst_sys_n_i      : in  std_logic;

      pps_csync_i      : in  std_logic;
      rf_reset_nco_i   : in  std_logic;

      dds_sync_p_o     : out std_logic;
      dds_sync_n_o     : out std_logic;        

      wb_slave_i       : in  t_wishbone_slave_in := cc_dummy_slave_in;
      wb_slave_o       : out t_wishbone_slave_out );

   attribute keep_hierarchy              : STRING;
   attribute keep_hierarchy of wr2rf_dds : entity is "yes";
   
end entity;

architecture rtl of wr2rf_dds is

   signal dds_sync_prebuf     : std_logic;
   signal clka_sync_prebuf    : std_logic;
   signal clkb_sync_prebuf    : std_logic;
   signal dds_ioupdate_prebuf : std_logic;
   
begin

   g_not_fine_delay : if (g_use_fine_delay = false) generate
   begin
      
      dds_sync_i : obufds
         generic map(
            slew => "FAST" )
         port map(
            o    => dds_sync_p_o,
            ob   => dds_sync_n_o,
            i    => rf_reset_nco_i );
      
   end generate;

   g_fine_delay : if (g_use_fine_delay = true) generate
   begin
      
      xwb_fine_pulse_gen_i : entity work.xwb_fine_pulse_gen
         generic map
         (
            g_use_odelay                => "111100",
            g_num_channels              => 4,
            g_use_external_serdes_clock => false,
            g_target_platform           => "Kintex7" )
         port map (
            clk_sys_i       => clk_sys_i,
            clk_ref_i       => clk_ref_i,
            rst_sys_n_i     => rst_sys_n_i,
            pps_p_i         => pps_csync_i,
            
            clk_ser_ext_i   => '0',
            ext_trigger_p_i => rf_reset_nco_i,
            
            pulse_o(0)      => dds_sync_prebuf,
            pulse_o(1)      => clka_sync_prebuf,
            pulse_o(2)      => clkb_sync_prebuf,
            pulse_o(3)      => dds_ioupdate_prebuf,

            slave_i         => wb_slave_i,
            slave_o         => wb_slave_o );

      
      dds_sync_i : obufds
         generic map(
            slew => "FAST" )
         port map(
            o    => dds_sync_p_o,
            ob   => dds_sync_n_o,
            i    => dds_sync_prebuf );
      
   end generate;

end architecture;
