--------------------------------------------------------------------------------
-- CERN BE-CO-HT
-- WR2RF_VME core
-- https://ohwr.org/project/vme-rf-wr-bobr
--------------------------------------------------------------------------------
--
-- unit name:   wr2rf_regs_core
--
-- description:
--
--------------------------------------------------------------------------------
-- Copyright (c) 2019 CERN (home.cern)
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.wishbone_pkg.all;

entity wr2rf_regs_core is

    generic (
        g_svec_mockup   : boolean := false;
        g_ref_equal_sys : boolean := true );
    port (
        clk_sys_i   : in  std_logic;
        rst_sys_n_i : in  std_logic;

        clk_ref_i   : in  std_logic;
        rst_ref_n_i : in  std_logic;

        --  Top-level wishbone port (from VME)
        wb_vme_i           : in    t_wishbone_slave_in;
        wb_vme_o           : out   t_wishbone_slave_out;

        --  Wishbone port for WR core
        wb_wrc_i           : in    t_wishbone_master_in;
        wb_wrc_o           : out   t_wishbone_master_out;

        --  Wishbone port for the VTUs
        wb_rf1_vtus_i      : in    t_wishbone_master_in;
        wb_rf1_vtus_o      : out   t_wishbone_master_out;

        wb_rf2_vtus_i      : in    t_wishbone_master_in;
        wb_rf2_vtus_o      : out   t_wishbone_master_out;

        wb_rfframerxtx_i   : in    t_wishbone_master_in;
        wb_rfframerxtx_o   : out   t_wishbone_master_out;

        wb_dds_fpg_i       : in    t_wishbone_master_in;
        wb_dds_fpg_o       : out   t_wishbone_master_out;

        wb_rf1_rfnco_i     : in    t_wishbone_master_in;
        wb_rf1_rfnco_o     : out   t_wishbone_master_out;

        wb_rf2_rfnco_i     : in    t_wishbone_master_in;
        wb_rf2_rfnco_o     : out   t_wishbone_master_out;        

        mmcm_locked_i    : in  std_logic;
        clk_sel_o        : out std_logic;
        clk_sys_reset_o  : out std_logic;
        wrcore_reset_n_o : out std_logic;
        iodelay_reset_o  : out std_logic;

        mmcm_shift_incdec_o : out std_logic;
        mmcm_shift_en_o     : out std_logic;
        mmcm_shift_busy_i   : in  std_logic;

        tmg_clk_term_en_o : out std_logic_vector(2 downto 1);
        tmg_io_term_en_o  : out std_logic_vector(4 downto 1);
        tmg_clk_oe_o      : out std_logic_vector(1 downto 0);
        tmg_io_oe_o       : out std_logic_vector(4 downto 1);
        ext_ref_dir_o     : out std_logic;

        dds_cs_n_o         : out std_logic;
        rf1_iqdac_cs_n_o   : out std_logic;
        rf2_iqdac_cs_n_o   : out std_logic;
        rf_sclk_o          : out std_logic;
        rf_sdi_o           : out std_logic;
        rf_sdo_i           : in  std_logic;

        dds_reset_o      : out std_logic;
        dds_ioupdate_o   : out std_logic;
        dds_profile_o    : out std_logic_vector(2 downto 0);
        dds_sync_error_i : in  std_logic;

        ocxo_enabled_o   : out std_logic;
        ocxo_uptime_i    : in  std_logic_vector(15 downto 0);

        pll_main_cs_n_o : out std_logic;
        pll_main_sclk_o : out std_logic;
        pll_main_sdi_o  : out std_logic;
        pll_main_sdo_i  : in  std_logic;

        pll_main_error_i     : in  std_logic;
        pll_main_locked_i    : in  std_logic;
        pll_main_sync_o      : out std_logic;

        wr2rfsd_nco_azimuthal_o  : out std_logic_vector(47 downto 0);
        wr2rfsd_nco_cabledelay_o : out std_logic_vector(47 downto 0);
        wr2rfsd_nco_hb_o         : out std_logic_vector(9  downto 0);
        loc_nco_ctrl_reset_o     : out std_logic;
        loc_nco_h1_ftw_o         : out std_logic_vector(47 downto 0);
        loc_nco_h1_prog_o        : out std_logic_vector(47 downto 0);
        loc_nco_update_valid_o   : out std_logic;
        loc_or_wrs_params_sel_o  : out std_logic;

        -- registers to ctrl/cfg the svec_wr2rf_vme mockup card
        svec_mup_gpio_sel_o       : out std_logic_vector(3 downto 0);
        svec_mup_led_sel_o        : out std_logic_vector(3 downto 0);
        svec_mup_bclk_rfclk_sel_o : out std_logic;
        svec_mup_ila_sel_o        : out std_logic;

        svec_mup_rftrig_t1stop_o  : out std_logic;
        svec_mup_rftrig_t1start_o : out std_logic;
        svec_mup_rftrig_t2stop_o  : out std_logic;
        svec_mup_rftrig_t2start_o : out std_logic;

        svec_mup_fmc1_term_o : out std_logic_vector(4 downto 0);
        svec_mup_fmc1_oe_n_o : out std_logic_vector(4 downto 0);
        svec_mup_fmc1_led_o  : out std_logic_vector(1 downto 0);
        svec_mup_fmc2_term_o : out std_logic_vector(4 downto 0);
        svec_mup_fmc2_oe_n_o : out std_logic_vector(4 downto 0);
        svec_mup_fmc2_led_o  : out std_logic_vector(1 downto 0);

        --  RF 1
        rf1_mux_sel_o        : out std_logic_vector(1 downto 0);
        rf1_mixer_en_o       : out std_logic;
        rf1_t1_delay_latch_o : out std_logic;
        rf1_t1_delay_oen_o   : out std_logic;
        rf1_t1_mux_sel_o     : out std_logic;
        rf1_t2_delay_latch_o : out std_logic;
        rf1_t2_delay_oen_o   : out std_logic;
        rf1_t2_mux_sel_o     : out std_logic;

        --  RF 2
        rf2_mux_sel_o        : out std_logic_vector(1 downto 0);
        rf2_mixer_en_o       : out std_logic;
        rf2_t1_delay_latch_o : out std_logic;
        rf2_t1_delay_oen_o   : out std_logic;
        rf2_t1_mux_sel_o     : out std_logic;
        rf2_t2_delay_latch_o : out std_logic;
        rf2_t2_delay_oen_o   : out std_logic;
        rf2_t2_mux_sel_o     : out std_logic;

        rf_delay_o           : out std_logic_vector(9 downto 0);

        flash_cs_n_o         : out  std_logic;
        flash_mosi_o         : out  std_logic;
        flash_miso_i         : in   std_logic;
        flash_sck_o          : out std_logic;

        tm_tai_i            : in  std_logic_vector(39 downto 0);
        tm_cycles_i         : in  std_logic_vector(27 downto 0);
        tm_linkup_i         : in  std_logic;
        tm_time_valid_i     : in  std_logic;
        pps_i               : in  std_logic := '0' );

end;

architecture arch of wr2rf_regs_core is
   
    signal wb_pll_spi_in          : t_wishbone_master_in;
    signal wb_pll_spi_out         : t_wishbone_master_out;

    signal wb_rf_spi_in           : t_wishbone_master_in;
    signal wb_rf_spi_out          : t_wishbone_master_out;

    signal wb_rf_in               : t_wishbone_master_in;
    signal wb_rf_out              : t_wishbone_master_out;

    signal wb16_wrpc_in           : t_wishbone_master_in;
    signal wb16_wrpc_out          : t_wishbone_master_out;

    signal wb_wrc_out             : t_wishbone_master_out;

    signal wb_fw_update_in        : t_wishbone_master_in;
    signal wb_fw_update_out       : t_wishbone_master_out;

    signal wb16_framerxtx_sys_in  : t_wishbone_master_in;
    signal wb16_framerxtx_sys_out : t_wishbone_master_out;
    signal wb16_framerxtx_ref_in  : t_wishbone_master_in;
    signal wb16_framerxtx_ref_out : t_wishbone_master_out;

    signal wb16_rf1_rfnco_in      : t_wishbone_master_in;
    signal wb16_rf1_rfnco_out     : t_wishbone_master_out;
    signal wb16_rf2_rfnco_in      : t_wishbone_master_in;
    signal wb16_rf2_rfnco_out     : t_wishbone_master_out;
    
    signal rf1_t1_delay_data : std_logic_vector (15 downto 0);
    signal rf1_t2_delay_data : std_logic_vector (15 downto 0);
    signal rf2_t1_delay_data : std_logic_vector (15 downto 0);
    signal rf2_t2_delay_data : std_logic_vector (15 downto 0);
    signal rf1_t1_delay_wr   : std_logic;
    signal rf1_t2_delay_wr   : std_logic;
    signal rf2_t1_delay_wr   : std_logic;
    signal rf2_t2_delay_wr   : std_logic;

    --  clk_ref based.
    signal wrs_frame_valid     : std_logic;
    signal wrs_frame_ftw       : std_logic_vector (47 downto 0);
    signal wrs_frame_last_ftw  : std_logic_vector (47 downto 0);
    signal wrs_frame_last_ctrl : std_logic_vector (15 downto 0);
    signal wrs_frame_counter   : std_logic_vector (15 downto 0);

    signal wrc_page_0, wrc_page_1, wrc_page : std_logic_vector(31 downto 0);
begin

    inst_wr2rf_regs: entity work.wr2rf_vme_regs
      port map (
        rst_n_i          => rst_sys_n_i,
        clk_i            => clk_sys_i,
        wb_i             => wb_vme_i,
        wb_o             => wb_vme_o,
        ctrl_rf1_vtus_i  => wb_rf1_vtus_i,
        ctrl_rf1_vtus_o  => wb_rf1_vtus_o,
        ctrl_rf2_vtus_i  => wb_rf2_vtus_i,
        ctrl_rf2_vtus_o  => wb_rf2_vtus_o,
        ctrl_rf1_rfnco_i => wb16_rf1_rfnco_in,
        ctrl_rf1_rfnco_o => wb16_rf1_rfnco_out,
        ctrl_rf2_rfnco_i => wb16_rf2_rfnco_in,
        ctrl_rf2_rfnco_o => wb16_rf2_rfnco_out,        

        init_hwver_i => x"ff01",

        init_clock_status_mmcm_locked_i => mmcm_locked_i,
        init_clock_ctrl_clk_sel_o       => clk_sel_o,
        init_clock_ctrl_mmcm_reset_o    => clk_sys_reset_o,

        init_wrcore_ctrl_reset_n_o      => wrcore_reset_n_o,
        init_iodelay_ctrl_reset_o       => iodelay_reset_o,

        init_mmcm_shift_incdec_o        => mmcm_shift_incdec_o,
        init_mmcm_shift_wr_o            => mmcm_shift_en_o,
        init_clock_status_shift_busy_i  => mmcm_shift_busy_i,

        init_tmg_io_term_o    => tmg_io_term_en_o,
        init_tmg_clk_term_o   => tmg_clk_term_en_o,
        init_tmg_clk_oe_o     => tmg_clk_oe_o,
        init_tmg_io_oe_o      => tmg_io_oe_o,
        init_pin_ctrl_ext_ref_dir_o => ext_ref_dir_o,

        init_dds_ctrl_reset_o        => dds_reset_o,
        init_dds_ctrl_ioupdate_o     => dds_ioupdate_o,
        init_dds_ctrl_profile_o      => dds_profile_o,
        init_dds_status_sync_error_i => dds_sync_error_i,

        init_ocxo_ctrl_enable_o      => ocxo_enabled_o,
        init_ocxo_uptime_val_i       => ocxo_uptime_i,        

        init_wrc_page0_o     => wrc_page_0,
        init_wrc_page1_o     => wrc_page_1,

        init_pll_status_error_i  => pll_main_error_i,
        init_pll_status_locked_i => pll_main_locked_i,
        init_pll_ctrl_sync_o     => pll_main_sync_o,

        init_nco_azimuthal_value_o       => wr2rfsd_nco_azimuthal_o,
        init_nco_cabledelay_value_o      => wr2rfsd_nco_cabledelay_o,
        init_nco_hb_value_o              => wr2rfsd_nco_hb_o,
        init_nco_h1_ftw_value_o          => loc_nco_h1_ftw_o,
        init_nco_h1_prog_value_o         => loc_nco_h1_prog_o,
        init_nco_ctrl_reset_nco_o        => loc_nco_ctrl_reset_o,
        init_nco_ctrl_reset_slip_o       => open,
        init_nco_ctrl_reset_fsk_o        => open,
        init_nco_ctrl_rate_o             => open,
        init_nco_update_valid_o          => loc_nco_update_valid_o,
        init_nco_loc_or_wrs_params_sel_o => loc_or_wrs_params_sel_o,

        init_svec_mup_ctrl_gpio_sel_o       => svec_mup_gpio_sel_o,
        init_svec_mup_ctrl_led_sel_o        => svec_mup_led_sel_o,
        init_svec_mup_ctrl_bclk_rfclk_sel_o => svec_mup_bclk_rfclk_sel_o,
        init_svec_mup_ctrl_ila_sel_o        => svec_mup_ila_sel_o,

        init_svec_mup_rftrig_t1stop_o  => svec_mup_rftrig_t1stop_o,
        init_svec_mup_rftrig_t1start_o => svec_mup_rftrig_t1start_o,
        init_svec_mup_rftrig_t2stop_o  => svec_mup_rftrig_t2stop_o,
        init_svec_mup_rftrig_t2start_o => svec_mup_rftrig_t2start_o,

        init_svec_mup_fmc1_term_o => svec_mup_fmc1_term_o,
        init_svec_mup_fmc1_oe_n_o => svec_mup_fmc1_oe_n_o,
        init_svec_mup_fmc1_led_o  => svec_mup_fmc1_led_o,
        init_svec_mup_fmc2_term_o => svec_mup_fmc2_term_o,
        init_svec_mup_fmc2_oe_n_o => svec_mup_fmc2_oe_n_o,
        init_svec_mup_fmc2_led_o  => svec_mup_fmc2_led_o,

        init_wrc_tai_value_i         => tm_tai_i,
        init_wrc_cycles_value_i      => tm_cycles_i,
        init_wrc_status_linkup_i     => tm_linkup_i,
        init_wrc_status_time_valid_i => tm_time_valid_i,

        init_rxframe_ftw_value_i => wrs_frame_last_ftw,
        init_rxframe_ctrl_i      => wrs_frame_last_ctrl,
        init_rxframe_counter_i   => wrs_frame_counter,

        init_pll_spi_i => wb_pll_spi_in,
        init_pll_spi_o => wb_pll_spi_out,

        init_rf_spi_i => wb_rf_spi_in,
        init_rf_spi_o => wb_rf_spi_out,

        init_fw_update_i => wb_fw_update_in,
        init_fw_update_o => wb_fw_update_out,

        init_rf_i => wb_rf_in,
        init_rf_o => wb_rf_out,

        init_framerxtx_i => wb16_framerxtx_sys_in,
        init_framerxtx_o => wb16_framerxtx_sys_out,

        init_wrpc_i => wb16_wrpc_in,
        init_wrpc_o => wb16_wrpc_out
      );


    inst_rf_spi: entity work.xwb_spi16
      generic map (
        g_interface_mode => CLASSIC,
        g_divider_len    => 8,
        g_max_char_len   => 128,
        g_num_slaves     => 3
      )
      port map (
        clk_sys_i        => clk_sys_i,
        rst_n_i          => rst_sys_n_i,
        slave_i          => wb_rf_spi_out,
        slave_o          => wb_rf_spi_in,
        int_o            => open,  --  TODO: interrupt
        pad_cs_o (0)     => dds_cs_n_o,
        pad_cs_o (1)     => rf1_iqdac_cs_n_o,
        pad_cs_o (2)     => rf2_iqdac_cs_n_o,
        pad_sclk_o       => rf_sclk_o,
        pad_mosi_o       => rf_sdi_o,
        pad_miso_i       => rf_sdo_i
      );

    inst_pll_spi: entity work.xwb_spi16
      generic map (
        g_interface_mode => CLASSIC,
        g_divider_len    => 8,
        g_max_char_len   => 16,
        g_num_slaves     => 1
      )
      port map (
        clk_sys_i        => clk_sys_i,
        rst_n_i          => rst_sys_n_i,
        slave_i          => wb_pll_spi_out,
        slave_o          => wb_pll_spi_in,
        int_o            => open,  --  TODO: interrupt
        pad_cs_o (0)     => pll_main_cs_n_o,
        pad_sclk_o       => pll_main_sclk_o,
        pad_mosi_o       => pll_main_sdi_o,
        pad_miso_i       => pll_main_sdo_i
      );

    my_inst: entity work.wr2rf_init_rf_regs
      port map (
        clk_i                    => clk_sys_i,
        rst_n_i                  => rst_sys_n_i,
        wb_i                     => wb_rf_out,
        wb_o                     => wb_rf_in,
        rf1_common_mixer_en_o    => rf1_mixer_en_o,
        rf1_common_mux_sel_o     => rf1_mux_sel_o,
        rf1_ch_0_csr_delay_oen_o => rf1_t1_delay_oen_o,
        rf1_ch_0_csr_mux_sel_o   => rf1_t1_mux_sel_o,
        rf1_ch_0_delay_o         => rf1_t1_delay_data,
        rf1_ch_0_delay_wr_o      => rf1_t1_delay_wr,
        rf1_ch_1_csr_delay_oen_o => rf1_t2_delay_oen_o,
        rf1_ch_1_csr_mux_sel_o   => rf1_t2_mux_sel_o,
        rf1_ch_1_delay_o         => rf1_t2_delay_data,
        rf1_ch_1_delay_wr_o      => rf1_t2_delay_wr,
        rf2_common_mixer_en_o    => rf2_mixer_en_o,
        rf2_common_mux_sel_o     => rf2_mux_sel_o,
        rf2_ch_0_csr_delay_oen_o => rf2_t1_delay_oen_o,
        rf2_ch_0_csr_mux_sel_o   => rf2_t1_mux_sel_o,
        rf2_ch_0_delay_o         => rf2_t1_delay_data,
        rf2_ch_0_delay_wr_o      => rf2_t1_delay_wr,
        rf2_ch_1_csr_delay_oen_o => rf2_t2_delay_oen_o,
        rf2_ch_1_csr_mux_sel_o   => rf2_t2_mux_sel_o,
        rf2_ch_1_delay_o         => rf2_t2_delay_data,
        rf2_ch_1_delay_wr_o      => rf2_t2_delay_wr );

    process(clk_sys_i)
    begin
      if rising_edge(clk_sys_i) then
        if rf1_t1_delay_wr    = '1' then
          rf_delay_o <= rf1_t1_delay_data(9 downto 0);
        elsif rf1_t2_delay_wr = '1' then
          rf_delay_o <= rf1_t2_delay_data(9 downto 0);
        elsif rf1_t2_delay_wr = '1' then
          rf_delay_o <= rf2_t1_delay_data(9 downto 0);
        elsif rf1_t2_delay_wr = '1' then
          rf_delay_o <= rf2_t2_delay_data(9 downto 0);
        else
          rf_delay_o <= (others => '0');
        end if;
        rf1_t1_delay_latch_o <= rf1_t1_delay_wr;
        rf1_t2_delay_latch_o <= rf1_t2_delay_wr;
        rf2_t1_delay_latch_o <= rf2_t1_delay_wr;
        rf2_t2_delay_latch_o <= rf2_t2_delay_wr;
      end if;
    end process;

    inst_wrc_wb16x32: entity work.wb16_to_wb32
      port map (
        clk_i   => clk_sys_i,
        rst_n_i => rst_sys_n_i,
        wb16_i  => wb16_wrpc_out,
        wb16_o  => wb16_wrpc_in,
        wb32_i  => wb_wrc_i,
        wb32_o  => wb_wrc_out );

    wrc_page <= wrc_page_0 when wb_wrc_out.adr(12) = '0' else wrc_page_1;

    --  Remap the wrc address to target only peripherals.
    wb_wrc_o <= (cyc => wb_wrc_out.cyc,
                 stb => wb_wrc_out.stb,
                 adr => (wb_wrc_out.adr and x"0000_0fff") or (wrc_page and x"ffff_f000"),
                 sel => wb_wrc_out.sel,
                 we  => wb_wrc_out.we,
                 dat => wb_wrc_out.dat);

    g_not_svec_mockup : if (g_svec_mockup = false) generate

       inst_wb_fw_update: entity work.xwb_xc7_fw_update
          port map (
             clk_i        => clk_sys_i,
             rst_n_i      => rst_sys_n_i,
             wb_i         => wb_fw_update_out,
             wb_o         => wb_fw_update_in,
             flash_cs_n_o => flash_cs_n_o,
             flash_mosi_o => flash_mosi_o,
             flash_miso_i => flash_miso_i,
             flash_sck_o  => flash_sck_o );

    end generate;

   --  RF Frame
    gen_ref_not_equal_sys : if (g_ref_equal_sys = false) generate
       
       inst_framerxtx_cross: entity work.xwb_clock_bridge
          generic map (
             g_SLAVE_PORT_WB_MODE  => CLASSIC,
             g_MASTER_PORT_WB_MODE => CLASSIC,
             g_SIZE                => 2)
          port map (
             slave_clk_i    => clk_sys_i,
             slave_rst_n_i  => rst_sys_n_i,
             slave_i        => wb16_framerxtx_sys_out,
             slave_o        => wb16_framerxtx_sys_in,
             master_clk_i   => clk_ref_i,
             master_rst_n_i => rst_ref_n_i,
             master_i       => wb16_framerxtx_ref_in,
             master_o       => wb16_framerxtx_ref_out );
       
    end generate;

    gen_ref_equal_sys : if (g_ref_equal_sys = true) generate    
       wb16_framerxtx_ref_in  <= wb16_framerxtx_sys_in;
       wb16_framerxtx_ref_out <= wb16_framerxtx_sys_out;       
    end generate;

    -- If this fails, add clock bridges for the rfx_rfnco blocks...
    assert g_ref_equal_sys severity failure;  
           
   inst_framerxtx_wb16x32: entity work.wb16_to_wb32
      port map (
        clk_i   => clk_ref_i,
        rst_n_i => rst_ref_n_i,
        wb16_i  => wb16_framerxtx_ref_out,
        wb16_o  => wb16_framerxtx_ref_in,
        wb32_i  => wb_rfframerxtx_i,
        wb32_o  => wb_rfframerxtx_o );

   inst_rf1_rfnco_wb16x32: entity work.wb16_to_wb32
      port map (
        clk_i   => clk_ref_i,
        rst_n_i => rst_ref_n_i,
        wb16_i  => wb16_rf1_rfnco_out,
        wb16_o  => wb16_rf1_rfnco_in,
        wb32_i  => wb_rf1_rfnco_i,
        wb32_o  => wb_rf1_rfnco_o );

   inst_rf2_rfnco_wb16x32: entity work.wb16_to_wb32
      port map (
        clk_i   => clk_ref_i,
        rst_n_i => rst_ref_n_i,
        wb16_i  => wb16_rf2_rfnco_out,
        wb16_o  => wb16_rf2_rfnco_in,
        wb32_i  => wb_rf2_rfnco_i,
        wb32_o  => wb_rf2_rfnco_o );
    

    -- fine pulse generator
    -- Need to do XXX

end arch;
