--------------------------------------------------------------------------------
-- CERN BE-CO-HT
--
-- https://www.ohwr.org/projects/wr2rf-vme
--------------------------------------------------------------------------------
--
-- unit name:   wr2rf_rftrigger_channel
--
-- description: Provides clock architecture wrapper for VTU
--
--------------------------------------------------------------------------------
-- Copyright CERN 2020
--------------------------------------------------------------------------------
-- Copyright and related rights are licensed under the Solderpad Hardware
-- License, Version 2.0 (the "License"); you may not use this file except
-- in compliance with the License. You may obtain a copy of the License at
-- http://solderpad.org/licenses/SHL-2.0.
-- Unless required by applicable law or agreed to in writing, software,
-- hardware and materials distributed under this License is distributed on an
-- "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
-- or implied. See the License for the specific language governing permissions
-- and limitations under the License.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

library unisim;
use unisim.vcomponents.all;

entity wr2rf_rftrigger_channel is

    port (
        rf_clk_p_i      : in  std_logic;
        rf_clk_n_i      : in  std_logic;

        rf_sync_i     : in  std_logic := '0';

        rf_trig_p_o     : out std_logic;
        rf_trig_n_o     : out std_logic;

        rf_trig_rst_p_o : out std_logic;
        rf_trig_rst_n_o : out std_logic;

        rf_clk_o        : out std_logic );

end entity;

architecture rtl of wr2rf_rftrigger_channel is

    signal rf_clk      : std_logic;
    signal rf_clk_r1   : std_logic; -- clock region 1
    signal rf_clk_r2   : std_logic; -- clock region 2
    signal rf_clk_mr   : std_logic;

    -- dummy logic
    signal r1_lfsr32   : std_logic_vector(31 downto 0);
    signal r1_lfsr32_r : std_logic_vector(31 downto 0) := X"01234567";
    signal r2_lfsr32   : std_logic_vector(31 downto 0);
    signal r2_lfsr32_r : std_logic_vector(31 downto 0) := X"76543210";

begin

    ibuf_1 : IBUFDS
        generic map (
            IOSTANDARD => "LVDS")
        port map (
            I  => rf_clk_p_i,
            IB => rf_clk_n_i,
            O  => rf_clk );

--    bufio_1 : BUFIO
--       port map (
--           I => rf_clk,
--           O => rf_clk_io );

    bufr_1 : BUFR
        generic map (
            SIM_DEVICE  => "7SERIES",
            BUFR_DIVIDE => "8" )
        port map (
            I   => rf_clk,
            O   => rf_clk_r1,
            CE  => '1',
            CLR => '0');

    bufmr_i : BUFMR
       port map (
           I => rf_clk,
           O => rf_clk_mr );

    bufr_2 : BUFR
        generic map (
            SIM_DEVICE  => "7SERIES",
            BUFR_DIVIDE => "8" )
        port map (
            I   => rf_clk_mr,
            O   => rf_clk_r2,
            CE  => '1',
            CLR => '0' );

    -- Just some dummy/placeholder code to instantiate the clock in region r1
    r1_lfsr32(0)           <= r1_lfsr32_r(31) xor r1_lfsr32_r(21) xor r1_lfsr32_r(1) xor r1_lfsr32_r(0);
    r1_lfsr32(31 downto 1) <= r1_lfsr32_r(30 downto 0);

    process (rf_clk_r1) is
    begin
        if rising_edge (rf_clk_r1) then
            if rf_sync_i = '1' then
                r1_lfsr32_r    <= r1_lfsr32;
            end if;
        end if;
    end process;

    rf_clk_o <= rf_clk_r1;


    -- Just some dummy/placeholder code to instantiate the clock in region r2
    r2_lfsr32(0)           <= r2_lfsr32_r(31) xor r2_lfsr32_r(21) xor r2_lfsr32_r(1) xor r2_lfsr32_r(0);
    r2_lfsr32(31 downto 1) <= r2_lfsr32_r(30 downto 0);

    process (rf_clk_r2) is
    begin
        if rising_edge (rf_clk_r2) then
            r2_lfsr32_r    <= r2_lfsr32;
        end if;
    end process;

    obuf_1 : OBUFDS
        generic map (
            IOSTANDARD => "LVDS" )
        port map (
            I  => r1_lfsr32_r(0),
            O  => rf_trig_p_o,
            OB => rf_trig_n_o );

    obuf_2 : OBUFDS
        generic map (
        IOSTANDARD => "LVDS" )
        port map (
            I  => r2_lfsr32_r(0),
            O  => rf_trig_rst_p_o,
            OB => rf_trig_rst_n_o );

end architecture;
