#ifndef __CHEBY__WR2RF_INIT_REGS__H__
#define __CHEBY__WR2RF_INIT_REGS__H__

#include "oc_spi16_regs.h"
#include "wr2rf_init_rf_regs.h"
#define WR2RF_INIT_REGS_SIZE 16384 /* 0x4000 = 16KB */

/* Hardware version */
#define WR2RF_INIT_REGS_HWVER 0x0UL

/* Control register for the system clock */
#define WR2RF_INIT_REGS_CLOCK_CTRL 0x2UL
#define WR2RF_INIT_REGS_CLOCK_CTRL_CLK_SEL 0x1UL
#define WR2RF_INIT_REGS_CLOCK_CTRL_MMCM_RESET 0x2UL

/* Control register for wrcore */
#define WR2RF_INIT_REGS_WRCORE_CTRL 0x4UL
#define WR2RF_INIT_REGS_WRCORE_CTRL_RESET_N 0x1UL

/* Control register for iodelayctrl */
#define WR2RF_INIT_REGS_IODELAY_CTRL 0x6UL
#define WR2RF_INIT_REGS_IODELAY_CTRL_RESET 0x1UL

/* Adjust the mmcm shift for the 10Mhz external clock */
#define WR2RF_INIT_REGS_MMCM_SHIFT 0x8UL
#define WR2RF_INIT_REGS_MMCM_SHIFT_INCDEC 0x1UL

/* Status register for system clock */
#define WR2RF_INIT_REGS_CLOCK_STATUS 0xaUL
#define WR2RF_INIT_REGS_CLOCK_STATUS_MMCM_LOCKED 0x1UL
#define WR2RF_INIT_REGS_CLOCK_STATUS_SHIFT_BUSY 0x2UL

/* timing io grouping of signals */
#define WR2RF_INIT_REGS_TMG 0xcUL
#define WR2RF_INIT_REGS_TMG_IO_TERM_MASK 0xfUL
#define WR2RF_INIT_REGS_TMG_IO_TERM_SHIFT 0
#define WR2RF_INIT_REGS_TMG_IO_OE_MASK 0xf0UL
#define WR2RF_INIT_REGS_TMG_IO_OE_SHIFT 4
#define WR2RF_INIT_REGS_TMG_CLK_TERM_MASK 0x300UL
#define WR2RF_INIT_REGS_TMG_CLK_TERM_SHIFT 8
#define WR2RF_INIT_REGS_TMG_CLK_OE_MASK 0xc00UL
#define WR2RF_INIT_REGS_TMG_CLK_OE_SHIFT 10

/* timing io grouping of signals */
#define WR2RF_INIT_REGS_PIN_CTRL 0xeUL
#define WR2RF_INIT_REGS_PIN_CTRL_EXT_REF_DIR 0x1UL

/* Status lines from the main PLL (LTC6950) */
#define WR2RF_INIT_REGS_PLL_STATUS 0x10UL
#define WR2RF_INIT_REGS_PLL_STATUS_ERROR 0x1UL
#define WR2RF_INIT_REGS_PLL_STATUS_LOCKED 0x2UL

/* Ctrl lines to the main PLL (LTC6950) */
#define WR2RF_INIT_REGS_PLL_CTRL 0x12UL
#define WR2RF_INIT_REGS_PLL_CTRL_SYNC 0x1UL

/* dds (AD9910) control signals */
#define WR2RF_INIT_REGS_DDS_CTRL 0x14UL
#define WR2RF_INIT_REGS_DDS_CTRL_RESET 0x1UL
#define WR2RF_INIT_REGS_DDS_CTRL_IOUPDATE 0x8UL
#define WR2RF_INIT_REGS_DDS_CTRL_PROFILE_MASK 0x70UL
#define WR2RF_INIT_REGS_DDS_CTRL_PROFILE_SHIFT 4

/* dds (AD9910) status signals */
#define WR2RF_INIT_REGS_DDS_STATUS 0x16UL
#define WR2RF_INIT_REGS_DDS_STATUS_SYNC_ERROR 0x1UL

/* ocxo configuration */
#define WR2RF_INIT_REGS_OCXO 0x18UL
#define WR2RF_INIT_REGS_OCXO_SIZE 4 /* 0x4 */

/* Enable the OCXO */
#define WR2RF_INIT_REGS_OCXO_CTRL 0x18UL
#define WR2RF_INIT_REGS_OCXO_CTRL_ENABLE 0x1UL

/* Records how long the ocxo has been enabled */
#define WR2RF_INIT_REGS_OCXO_UPTIME 0x1aUL
#define WR2RF_INIT_REGS_OCXO_UPTIME_VAL_MASK 0xffffUL
#define WR2RF_INIT_REGS_OCXO_UPTIME_VAL_SHIFT 0

/* NCO control */
#define WR2RF_INIT_REGS_NCO 0x40UL
#define WR2RF_INIT_REGS_NCO_SIZE 64 /* 0x40 */

/* Adjust NCO output for azimuthal position */
#define WR2RF_INIT_REGS_NCO_AZIMUTHAL 0x40UL
#define WR2RF_INIT_REGS_NCO_AZIMUTHAL_VALUE_MASK 0xffffffffffffULL
#define WR2RF_INIT_REGS_NCO_AZIMUTHAL_VALUE_SHIFT 0

/* Adjust NCO output for cable delays */
#define WR2RF_INIT_REGS_NCO_CABLEDELAY 0x48UL
#define WR2RF_INIT_REGS_NCO_CABLEDELAY_VALUE_MASK 0xffffffffffffULL
#define WR2RF_INIT_REGS_NCO_CABLEDELAY_VALUE_SHIFT 0

/* harmonic factor */
#define WR2RF_INIT_REGS_NCO_HB 0x50UL
#define WR2RF_INIT_REGS_NCO_HB_VALUE_MASK 0x3ffUL
#define WR2RF_INIT_REGS_NCO_HB_VALUE_SHIFT 0

/* Adjust NCO increment per clock cycle */
#define WR2RF_INIT_REGS_NCO_H1_FTW 0x58UL
#define WR2RF_INIT_REGS_NCO_H1_FTW_VALUE_MASK 0xffffffffffffULL
#define WR2RF_INIT_REGS_NCO_H1_FTW_VALUE_SHIFT 0

/* Adjust NCO starting phase */
#define WR2RF_INIT_REGS_NCO_H1_PROG 0x60UL
#define WR2RF_INIT_REGS_NCO_H1_PROG_VALUE_MASK 0xffffffffffffULL
#define WR2RF_INIT_REGS_NCO_H1_PROG_VALUE_SHIFT 0

/* control */
#define WR2RF_INIT_REGS_NCO_CTRL 0x68UL
#define WR2RF_INIT_REGS_NCO_CTRL_RESET_NCO 0x1UL
#define WR2RF_INIT_REGS_NCO_CTRL_RESET_SLIP 0x2UL
#define WR2RF_INIT_REGS_NCO_CTRL_RESET_FSK 0x4UL
#define WR2RF_INIT_REGS_NCO_CTRL_RATE_MASK 0x70UL
#define WR2RF_INIT_REGS_NCO_CTRL_RATE_SHIFT 4

/* Used to signal that all parameters are valid and ready for use */
#define WR2RF_INIT_REGS_NCO_UPDATE 0x6aUL
#define WR2RF_INIT_REGS_NCO_UPDATE_VALID 0x1UL

/* Used to determine if local registers (=1) or WRStreamer (=0) parameters are selected */
#define WR2RF_INIT_REGS_NCO_LOC_OR_WRS 0x6cUL
#define WR2RF_INIT_REGS_NCO_LOC_OR_WRS_PARAMS_SEL 0x1UL

/* SVEC mockup board registers */
#define WR2RF_INIT_REGS_SVEC_MUP 0x80UL
#define WR2RF_INIT_REGS_SVEC_MUP_SIZE 8 /* 0x8 */

/* Front panel select */
#define WR2RF_INIT_REGS_SVEC_MUP_CTRL 0x80UL
#define WR2RF_INIT_REGS_SVEC_MUP_CTRL_GPIO_SEL_MASK 0xfUL
#define WR2RF_INIT_REGS_SVEC_MUP_CTRL_GPIO_SEL_SHIFT 0
#define WR2RF_INIT_REGS_SVEC_MUP_CTRL_LED_SEL_MASK 0xf0UL
#define WR2RF_INIT_REGS_SVEC_MUP_CTRL_LED_SEL_SHIFT 4
#define WR2RF_INIT_REGS_SVEC_MUP_CTRL_BCLK_RFCLK_SEL 0x100UL
#define WR2RF_INIT_REGS_SVEC_MUP_CTRL_ILA_SEL 0x200UL

/* rf trigger mockup */
#define WR2RF_INIT_REGS_SVEC_MUP_RFTRIG 0x82UL
#define WR2RF_INIT_REGS_SVEC_MUP_RFTRIG_T1STOP 0x1UL
#define WR2RF_INIT_REGS_SVEC_MUP_RFTRIG_T1START 0x2UL
#define WR2RF_INIT_REGS_SVEC_MUP_RFTRIG_T2STOP 0x4UL
#define WR2RF_INIT_REGS_SVEC_MUP_RFTRIG_T2START 0x8UL

/* fmc dio */
#define WR2RF_INIT_REGS_SVEC_MUP_FMC1 0x84UL
#define WR2RF_INIT_REGS_SVEC_MUP_FMC1_TERM_MASK 0x1fUL
#define WR2RF_INIT_REGS_SVEC_MUP_FMC1_TERM_SHIFT 0
#define WR2RF_INIT_REGS_SVEC_MUP_FMC1_OE_N_MASK 0x3e0UL
#define WR2RF_INIT_REGS_SVEC_MUP_FMC1_OE_N_SHIFT 5
#define WR2RF_INIT_REGS_SVEC_MUP_FMC1_LED_MASK 0xc00UL
#define WR2RF_INIT_REGS_SVEC_MUP_FMC1_LED_SHIFT 10

/* fmc dio */
#define WR2RF_INIT_REGS_SVEC_MUP_FMC2 0x86UL
#define WR2RF_INIT_REGS_SVEC_MUP_FMC2_TERM_MASK 0x1fUL
#define WR2RF_INIT_REGS_SVEC_MUP_FMC2_TERM_SHIFT 0
#define WR2RF_INIT_REGS_SVEC_MUP_FMC2_OE_N_MASK 0x3e0UL
#define WR2RF_INIT_REGS_SVEC_MUP_FMC2_OE_N_SHIFT 5
#define WR2RF_INIT_REGS_SVEC_MUP_FMC2_LED_MASK 0xc00UL
#define WR2RF_INIT_REGS_SVEC_MUP_FMC2_LED_SHIFT 10

/* Last rx frame from WR */
#define WR2RF_INIT_REGS_RXFRAME 0x90UL
#define WR2RF_INIT_REGS_RXFRAME_SIZE 16 /* 0x10 */

/* Last FTW received */
#define WR2RF_INIT_REGS_RXFRAME_FTW 0x90UL
#define WR2RF_INIT_REGS_RXFRAME_FTW_VALUE_MASK 0xffffffffffffULL
#define WR2RF_INIT_REGS_RXFRAME_FTW_VALUE_SHIFT 0

/* Last Control received */
#define WR2RF_INIT_REGS_RXFRAME_CTRL 0x98UL

/* Number of FTW received */
#define WR2RF_INIT_REGS_RXFRAME_COUNTER 0x9aUL

/* WR status and time */
#define WR2RF_INIT_REGS_WRC 0xa0UL
#define WR2RF_INIT_REGS_WRC_SIZE 32 /* 0x20 */

/* Absolute time (in sec) */
#define WR2RF_INIT_REGS_WRC_TAI 0xa0UL
#define WR2RF_INIT_REGS_WRC_TAI_VALUE_MASK 0xffffffffffULL
#define WR2RF_INIT_REGS_WRC_TAI_VALUE_SHIFT 0

/* Number of cycles */
#define WR2RF_INIT_REGS_WRC_CYCLES 0xa8UL
#define WR2RF_INIT_REGS_WRC_CYCLES_VALUE_MASK 0xfffffffUL
#define WR2RF_INIT_REGS_WRC_CYCLES_VALUE_SHIFT 0

/* WR status */
#define WR2RF_INIT_REGS_WRC_STATUS 0xacUL
#define WR2RF_INIT_REGS_WRC_STATUS_LINKUP 0x1UL
#define WR2RF_INIT_REGS_WRC_STATUS_TIME_VALID 0x2UL

/* Page 0 offset */
#define WR2RF_INIT_REGS_WRC_PAGE0 0xb0UL

/* Page 1 offset */
#define WR2RF_INIT_REGS_WRC_PAGE1 0xb4UL

/* SPI master for the main PLL (LTC6950) */
#define WR2RF_INIT_REGS_PLL_SPI 0xc0UL
#define WR2RF_INIT_REGS_PLL_SPI_SIZE 32 /* 0x20 */

/* SPI master for RF chips.
cs0: DDS
cs1: IQ DAC 1
cs2: IQ DAC 2
 */
#define WR2RF_INIT_REGS_RF_SPI 0xe0UL
#define WR2RF_INIT_REGS_RF_SPI_SIZE 32 /* 0x20 */

/* Firmware update (FAR) */
#define WR2RF_INIT_REGS_FW_UPDATE 0x100UL
#define WR2RF_INIT_REGS_FW_UPDATE_SIZE 2 /* 0x2 */

/* Register for RF components */
#define WR2RF_INIT_REGS_RF 0x120UL
#define WR2RF_INIT_REGS_RF_SIZE 32 /* 0x20 */

/* RF Frame transceiver */
#define WR2RF_INIT_REGS_FRAMERXTX 0x140UL
#define WR2RF_INIT_REGS_FRAMERXTX_SIZE 64 /* 0x40 */

/* WR ptp core (peripheral only) */
#define WR2RF_INIT_REGS_WRPC 0x2000UL
#define WR2RF_INIT_REGS_WRPC_SIZE 8192 /* 0x2000 = 8KB */

struct wr2rf_init_regs {
  /* [0x0]: REG (ro) Hardware version */
  uint16_t hwver;

  /* [0x2]: REG (rw) Control register for the system clock */
  uint16_t clock_ctrl;

  /* [0x4]: REG (rw) Control register for wrcore */
  uint16_t wrcore_ctrl;

  /* [0x6]: REG (rw) Control register for iodelayctrl */
  uint16_t iodelay_ctrl;

  /* [0x8]: REG (wo) Adjust the mmcm shift for the 10Mhz external clock */
  uint16_t mmcm_shift;

  /* [0xa]: REG (ro) Status register for system clock */
  uint16_t clock_status;

  /* [0xc]: REG (rw) timing io grouping of signals */
  uint16_t tmg;

  /* [0xe]: REG (rw) timing io grouping of signals */
  uint16_t pin_ctrl;

  /* [0x10]: REG (ro) Status lines from the main PLL (LTC6950) */
  uint16_t pll_status;

  /* [0x12]: REG (rw) Ctrl lines to the main PLL (LTC6950) */
  uint16_t pll_ctrl;

  /* [0x14]: REG (rw) dds (AD9910) control signals */
  uint16_t dds_ctrl;

  /* [0x16]: REG (ro) dds (AD9910) status signals */
  uint16_t dds_status;

  /* [0x18]: BLOCK ocxo configuration */
  struct ocxo {
    /* [0x0]: REG (rw) Enable the OCXO */
    uint16_t ctrl;

    /* [0x2]: REG (ro) Records how long the ocxo has been enabled */
    uint16_t uptime;
  } ocxo;

  /* padding to: 16 words */
  uint32_t __padding_0[9];

  /* [0x40]: BLOCK NCO control */
  struct nco {
    /* [0x0]: REG (rw) Adjust NCO output for azimuthal position */
    uint64_t azimuthal;

    /* [0x8]: REG (rw) Adjust NCO output for cable delays */
    uint64_t cabledelay;

    /* [0x10]: REG (rw) harmonic factor */
    uint16_t hb;

    /* padding to: 24 words */
    uint8_t __padding_0[6];

    /* [0x18]: REG (rw) Adjust NCO increment per clock cycle */
    uint64_t h1_ftw;

    /* [0x20]: REG (rw) Adjust NCO starting phase */
    uint64_t h1_prog;

    /* [0x28]: REG (rw) control */
    uint16_t ctrl;

    /* [0x2a]: REG (rw) Used to signal that all parameters are valid and ready for use */
    uint16_t update;

    /* [0x2c]: REG (rw) Used to determine if local registers (=1) or WRStreamer (=0) parameters are selected */
    uint16_t loc_or_wrs;

    /* padding to: 44 words */
    uint8_t __padding_1[18];
  } nco;

  /* [0x80]: BLOCK SVEC mockup board registers */
  struct svec_mup {
    /* [0x0]: REG (rw) Front panel select */
    uint16_t ctrl;

    /* [0x2]: REG (rw) rf trigger mockup */
    uint16_t rftrig;

    /* [0x4]: REG (rw) fmc dio */
    uint16_t fmc1;

    /* [0x6]: REG (rw) fmc dio */
    uint16_t fmc2;
  } svec_mup;

  /* padding to: 36 words */
  uint32_t __padding_1[2];

  /* [0x90]: BLOCK Last rx frame from WR */
  struct rxframe {
    /* [0x0]: REG (ro) Last FTW received */
    uint64_t ftw;

    /* [0x8]: REG (ro) Last Control received */
    uint16_t ctrl;

    /* [0xa]: REG (ro) Number of FTW received */
    uint16_t counter;

    /* padding to: 2 words */
    uint32_t __padding_0[1];
  } rxframe;

  /* [0xa0]: BLOCK WR status and time */
  struct wrc {
    /* [0x0]: REG (ro) Absolute time (in sec) */
    uint64_t tai;

    /* [0x8]: REG (ro) Number of cycles */
    uint32_t cycles;

    /* [0xc]: REG (ro) WR status */
    uint16_t status;

    /* padding to: 16 words */
    uint8_t __padding_0[2];

    /* [0x10]: REG (rw) Page 0 offset */
    uint32_t page0;

    /* [0x14]: REG (rw) Page 1 offset */
    uint32_t page1;

    /* padding to: 5 words */
    uint32_t __padding_1[2];
  } wrc;

  /* [0xc0]: SUBMAP SPI master for the main PLL (LTC6950) */
  struct oc_spi16_regs pll_spi;

  /* padding to: 224 words */
  uint8_t __padding_2[6];

  /* [0xe0]: SUBMAP SPI master for RF chips.
cs0: DDS
cs1: IQ DAC 1
cs2: IQ DAC 2
 */
  struct oc_spi16_regs rf_spi;

  /* padding to: 256 words */
  uint8_t __padding_3[6];

  /* [0x100]: SUBMAP Firmware update (FAR) */
  uint8_t fw_update[2];

  /* padding to: 288 words */
  uint8_t __padding_4[30];

  /* [0x120]: SUBMAP Register for RF components */
  struct wr2rf_init_rf_regs rf;

  /* [0x140]: SUBMAP RF Frame transceiver */
  uint32_t framerxtx[16];

  /* padding to: 2048 words */
  uint32_t __padding_5[1952];

  /* [0x2000]: SUBMAP WR ptp core (peripheral only) */
  uint32_t wrpc[2048];
};

#endif /* __CHEBY__WR2RF_INIT_REGS__H__ */
