#ifndef __CHEBY__RFNCO__H__
#define __CHEBY__RFNCO__H__

#include "ipInfo.h"
#define RFNCO_SIZE 4096 /* 0x1000 = 4KB */
#define RFNCO_MEMMAP_VERSION 0x1UL
#define RFNCO_IDENT 0xffUL

/* None */
#define RFNCO_IPINFO 0x0UL
#define RFNCO_IPINFO_SIZE 32 /* 0x20 */

/* Frequency Tuning Words and values loaded at softLoad or hw inputs valid */
#define RFNCO_FTW 0x80UL
#define RFNCO_FTW_SIZE 128 /* 0x80 */

/* Harmonic 1 FTW */
#define RFNCO_FTW_H1 0x80UL

/* Cavity on H1 FTW */
#define RFNCO_FTW_ON 0x88UL

/* LO 0 FTW */
#define RFNCO_FTW_LO0 0x90UL

/* LO 1 FTW */
#define RFNCO_FTW_LO1 0x98UL

/* H1 phase offset (Azimuth) */
#define RFNCO_FTW_OFFSETH1 0xa0UL

/* FSK rate */
#define RFNCO_FTW_RATE 0xa8UL

/* Harmonic number */
#define RFNCO_FTW_H 0xacUL

/* offset from Frev for FSK */
#define RFNCO_FTW_OFFSETFSK 0xb0UL

/* slip H1 FTW */
#define RFNCO_FTW_SLIP 0xb8UL

/* Output phases readout */
#define RFNCO_PHASES 0x100UL
#define RFNCO_PHASES_SIZE 128 /* 0x80 */

/* H1 phase with compensation */
#define RFNCO_PHASES_H1 0x100UL

/* H1 phase direct */
#define RFNCO_PHASES_H1MAIN 0x108UL

/* H1 slippage phase */
#define RFNCO_PHASES_H1SLIP 0x110UL

/* IF0 phase */
#define RFNCO_PHASES_IF0 0x118UL

/* IF1 phase */
#define RFNCO_PHASES_IF1 0x120UL

/* control register */
#define RFNCO_CONTROL 0x180UL
#define RFNCO_CONTROL_RESET 0x1UL
#define RFNCO_CONTROL_SOFTLOAD 0x2UL
#define RFNCO_CONTROL_SELFTWLO0 0x4UL
#define RFNCO_CONTROL_SELFTWLO1 0x8UL
#define RFNCO_CONTROL_SELOFFSETH1 0x10UL
#define RFNCO_CONTROL_SELOFFSETFSKH1 0x20UL
#define RFNCO_CONTROL_SELRATE 0x40UL
#define RFNCO_CONTROL_SELFTWH1 0x80UL
#define RFNCO_CONTROL_SELFTWH1ON 0x100UL
#define RFNCO_CONTROL_SELWRFTWH1 0x200UL
#define RFNCO_CONTROL_SELWRDFTWH1SLIP 0x400UL
#define RFNCO_CONTROL_SELDFTWH1SLIP 0x800UL
#define RFNCO_CONTROL_SLIPSIGN 0x1000UL
#define RFNCO_CONTROL_ENABLEFSK 0x4000UL
#define RFNCO_CONTROL_RESETFSK 0x8000UL
#define RFNCO_CONTROL_ENABLESLIP 0x10000UL
#define RFNCO_CONTROL_RESETSLIP 0x20000UL
#define RFNCO_CONTROL_SIGNIF0 0x40000UL
#define RFNCO_CONTROL_SIGNIF1 0x80000UL
#define RFNCO_CONTROL_EXTRESETENABLE 0x100000UL
#define RFNCO_CONTROL_LATCHPHASES 0x200000UL
#define RFNCO_CONTROL_SELLOAD 0x400000UL
#define RFNCO_CONTROL_ENABLEFREQDOUBLING 0x800000UL

/* status register */
#define RFNCO_STATUS 0x184UL
#define RFNCO_STATUS_INTERLOCK 0x1UL
#define RFNCO_STATUS_PHASEH1VLD 0x8UL
#define RFNCO_STATUS_AMAUTOON 0x40UL

/* latches the presence of pulses */
#define RFNCO_PRESENTLATCH 0x188UL
#define RFNCO_PRESENTLATCH_FREV 0x1UL
#define RFNCO_PRESENTLATCH_AM 0x2UL
#define RFNCO_PRESENTLATCH_TW 0x4UL
#define RFNCO_PRESENTLATCH_COEF 0x8UL
#define RFNCO_PRESENTLATCH_LOAD 0x10UL
#define RFNCO_PRESENTLATCH_EXTRESET 0x20UL

/* fixed delay for phase compensation in sec. */
#define RFNCO_DELAYCOMP 0x18cUL

/* readouts from NCO divider */
#define RFNCO_READBACK 0x1c0UL
#define RFNCO_READBACK_SIZE 64 /* 0x40 */

/* None */
#define RFNCO_READBACK_COEF 0x1c0UL

/* None */
#define RFNCO_READBACK_TW 0x1c4UL

/* None */
#define RFNCO_READBACK_PHASEH1COMP 0x1c8UL

struct rfnco {
  /* [0x0]: SUBMAP (no description) */
  struct ipInfo ipInfo;

  /* padding to: 32 words */
  uint32_t __padding_0[27];

  /* [0x80]: BLOCK Frequency Tuning Words and values loaded at softLoad or hw inputs valid */
  struct ftw {
    /* [0x0]: REG (rw) Harmonic 1 FTW */
    uint64_t h1;

    /* [0x8]: REG (rw) Cavity on H1 FTW */
    uint64_t on;

    /* [0x10]: REG (rw) LO 0 FTW */
    uint64_t lo0;

    /* [0x18]: REG (rw) LO 1 FTW */
    uint64_t lo1;

    /* [0x20]: REG (rw) H1 phase offset (Azimuth) */
    uint64_t offsetH1;

    /* [0x28]: REG (rw) FSK rate */
    uint32_t rate;

    /* [0x2c]: REG (rw) Harmonic number */
    uint32_t h;

    /* [0x30]: REG (rw) offset from Frev for FSK */
    uint64_t offsetFSK;

    /* [0x38]: REG (rw) slip H1 FTW */
    uint32_t slip;

    /* padding to: 14 words */
    uint32_t __padding_0[17];
  } ftw;

  /* [0x100]: BLOCK Output phases readout */
  struct phases {
    /* [0x0]: REG (ro) H1 phase with compensation */
    uint64_t h1;

    /* [0x8]: REG (ro) H1 phase direct */
    uint64_t h1Main;

    /* [0x10]: REG (ro) H1 slippage phase */
    uint64_t h1Slip;

    /* [0x18]: REG (ro) IF0 phase */
    uint64_t if0;

    /* [0x20]: REG (ro) IF1 phase */
    uint64_t if1;

    /* padding to: 8 words */
    uint32_t __padding_0[22];
  } phases;

  /* [0x180]: REG (rw) control register */
  uint32_t control;

  /* [0x184]: REG (ro) status register */
  uint32_t status;

  /* [0x188]: REG (rw) latches the presence of pulses */
  uint32_t presentLatch;

  /* [0x18c]: REG (rw) fixed delay for phase compensation in sec. */
  uint32_t delayComp;

  /* padding to: 112 words */
  uint32_t __padding_1[12];

  /* [0x1c0]: BLOCK readouts from NCO divider */
  struct readback {
    /* [0x0]: REG (ro) (no description) */
    uint32_t coef;

    /* [0x4]: REG (ro) (no description) */
    uint32_t tw;

    /* [0x8]: REG (ro) (no description) */
    uint64_t phaseH1Comp;

    /* padding to: 2 words */
    uint32_t __padding_0[12];
  } readback;

  /* padding to: 112 words */
  uint32_t __padding_2[896];
};

#endif /* __CHEBY__RFNCO__H__ */
