#include <stddef.h>
#include <assert.h>
#include "board.h"
#include "regs.h"
#include "oc_spi16.h"

uint32_t
libwr2rf_16x32_readl(unsigned off)
{
  uint32_t l, h;

  l = libwr2rf_readh (off);
  h = libwr2rf_readh (off + 2);
  return (h << 16) | l;
}

void
libwr2rf_16x32_writel(unsigned off, uint32_t val)
{
  libwr2rf_writeh (val >> 16, off + 2);
  libwr2rf_writeh (val & 0xffff, off);
}

uint32_t
libwr2rf_be_readl(unsigned off)
{
  uint32_t l, h;

  h = libwr2rf_readh (off);
  l = libwr2rf_readh (off + 2);
  return (h << 16) | l;
}

void
libwr2rf_be_writel(unsigned off, uint32_t val)
{
  libwr2rf_writeh (val >> 16, off);
  libwr2rf_writeh (val & 0xffff, off + 2);
}

void libwr2rf_pll_spi_init(void)
{
  /* Min period is 25+25ns, so max freq is 20Mhz.  */
  libwr2rf_oc_spi_init(PLL_BASE, 2);
}

uint8_t libwr2rf_pll_read(unsigned reg)
{
  uint32_t rval;
  libwr2rf_oc_spi_txrx16(PLL_BASE, PLL_CS, 16, (reg << 9) | (1 << 8), &rval);
  return rval & 0xff;
}

void libwr2rf_pll_write(unsigned reg, uint8_t value)
{
  libwr2rf_oc_spi_txrx16(PLL_BASE, PLL_CS, 16, (reg << 9) | (value & 0xff),
    NULL);
}

void libwr2rf_dds_spi_init(void)
{
  /* Min period is 25+25ns, so max freq is 20Mhz.  */
  libwr2rf_oc_spi_init(DDS_BASE, 2);
}

void libwr2rf_dds_write(unsigned reg, uint64_t value, unsigned nbits)
{
  uint16_t vals[8];

  assert (reg < 32);

  /* Note: valid nbits are: 64, 32, 16.   */
  assert (nbits == 16 || nbits == 32 || nbits == 64);

  vals[0] = value & 0xffff;
  vals[1] = (value >> 16) & 0xffff;
  vals[2] = (value >> 32) & 0xffff;
  vals[3] = (value >> 48) & 0xffff;
  vals[nbits / 16] = reg;

  libwr2rf_oc_spi_txrx128(DDS_BASE, DDS_CS, nbits + 8, vals);
}

uint64_t libwr2rf_dds_read(unsigned reg, unsigned nbits)
{
  uint16_t vals[8];
  uint64_t res;

  assert (reg < 32);

  /* Note: valid nbits are: 64, 32, 16.   */
  assert (nbits == 16 || nbits == 32 || nbits == 64);

  vals[0] = 0;
  vals[1] = 0;
  vals[2] = 0;
  vals[3] = 0;
  vals[nbits / 16] = 0x80 | reg;

  libwr2rf_oc_spi_txrx128(DDS_BASE, DDS_CS, nbits + 8, vals);

  res = (uint64_t)vals[0];
  if (nbits > 16) {
      res |= ((uint64_t)vals[1]) << 16;
  }
  if (nbits > 32) {
    res |= ((uint64_t)vals[2]) << 32;
    res |= ((uint64_t)vals[3]) << 48;
  }

  return res;
}

void libwr2rf_dds_ioupdate(void)
{
  libwr2rf_writeh(WR2RF_INIT_REGS_DDS_CTRL_IOUPDATE,
                  WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_DDS_CTRL);
}

unsigned libwr2rf_dds_sync_error(void)
{
  return libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_DDS_STATUS)
    & WR2RF_INIT_REGS_DDS_STATUS_SYNC_ERROR;
}

uint8_t libwr2rf_dac_read(unsigned dev, unsigned reg)
{
  uint32_t rval;
  libwr2rf_oc_spi_txrx16(DAC_BASE, dev, 16, 0x8000 | (reg << 8), &rval);
  return rval & 0xff;
}

void libwr2rf_dac_write(unsigned dev, unsigned reg, uint8_t value)
{
  libwr2rf_oc_spi_txrx16(DAC_BASE, dev, 16, (reg << 8) | (value & 0xff),
    NULL);
}
