#ifndef __BOARD_H_
#define __BOARD_H_

#include "io.h"

/* For the main pll (ltc6950).  */
void libwr2rf_pll_spi_init(void);
uint8_t libwr2rf_pll_read(unsigned reg);
void libwr2rf_pll_write(unsigned reg, uint8_t value);

/* For the dds (ad9910).  */
void libwr2rf_dds_spi_init(void);  /* Note: common to dds and dac.  */
void libwr2rf_dds_init(void);  /* In ad9910_init.c  */

/* Write and read a DDS register (using spi).  */
void libwr2rf_dds_write(unsigned reg, uint64_t value, unsigned nbits);
uint64_t libwr2rf_dds_read(unsigned reg, unsigned nbits);

/* Trigger ioupdate.  */
void libwr2rf_dds_ioupdate(void);

/* Read the sync_error input.  */
unsigned libwr2rf_dds_sync_error(void);

/* Read/write a DAC register.  DEV must be DAC1_CS or DAC2_CS.  */
uint8_t libwr2rf_dac_read(unsigned dev, unsigned reg);
void libwr2rf_dac_write(unsigned dev, unsigned reg, uint8_t value);

/* From dac_timing.c:  */
void libwr2rf_dac_init(unsigned dev);
int libwr2rf_dac_port_timing(unsigned dev);

/* Raw 32bit IOs through the wb16 to wb32 bridge.  */
uint32_t libwr2rf_16x32_readl(unsigned off);
void libwr2rf_16x32_writel(unsigned off, uint32_t val);

/* Regular BE 32bit IOs.  Not atomic.  */
uint32_t libwr2rf_be_readl(unsigned off);
void libwr2rf_be_writel(unsigned off, uint32_t val);

/* For init.c */
void libwr2rf_init(void);

#endif /* __BOARD_H_ */
