#include <stdlib.h>
#include <stdio.h>
#include "board.h"
#include "regs.h"

struct pll_reg_cfg {
  unsigned char reg;
  unsigned char val;
};

static const struct pll_reg_cfg pll_cfg[] = {
#include "ltc6950_config.h"
};

static void
init_ocxo(void)
{
    libwr2rf_writeh(WR2RF_INIT_REGS_OCXO_CTRL_ENABLE, WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_OCXO);
}

static void
init_ltc6950 (void)
{
  unsigned i;

  /* Full reset ?  */

  /* Set registers.  */
  for (i = 0; i < sizeof(pll_cfg)/sizeof(pll_cfg[0]); i++) {
    const struct pll_reg_cfg *cfg = &pll_cfg[i];
    libwr2rf_pll_write(cfg->reg, cfg->val);
  }

  /* Wait until locked.  */
  while (1) {
    unsigned v = libwr2rf_pll_read(0);
    if (v & 0x4)
      break;
  }
}

/* Select the DDS clock as clock source.  */
static void
libwr2rf_clock_select_dds (void)
{
    unsigned mmcm_locked      = 0;
    unsigned mmcm_retries     = 0;
    unsigned mmcm_max_retries = 100;
    unsigned ctrl;


    // take sys clock mmcm out of reset + then switch bufg mux select from dmtd to ltc6950 
    libwr2rf_writeh(0, WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_CLOCK_CTRL);

    // Wait for FPGA MMCM to lock...
    while (! mmcm_locked) {
        unsigned mmcm_status = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_CLOCK_STATUS);
        mmcm_locked = !!(mmcm_status & WR2RF_INIT_REGS_CLOCK_STATUS_MMCM_LOCKED);
        if (mmcm_retries > mmcm_max_retries) {
            printf("MMCM not locking on LTC6950 FPGA sys clock.");
            return;
        }
        mmcm_retries++;
    }

    libwr2rf_writeh(WR2RF_INIT_REGS_CLOCK_CTRL_CLK_SEL, WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_CLOCK_CTRL);
    ctrl = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_CLOCK_CTRL);

    printf("SysClockCtrl=%04x\n", ctrl);
}

static void
init_wrcore(void)
{
    /* Take the wrcore out of reset */
    libwr2rf_writeh(WR2RF_INIT_REGS_WRCORE_CTRL_RESET_N, 
                    WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRCORE_CTRL);
}

static void
init_iodelay(void)
{
    /* Apply a reset sequence to the iodelayctrl units */
    unsigned iodelay_rst;
    unsigned iodelay_addr = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_IODELAY_CTRL;

    iodelay_rst = libwr2rf_readh(iodelay_addr);
    iodelay_rst = !!(iodelay_rst & WR2RF_INIT_REGS_IODELAY_CTRL_RESET);

    /* Take the wrcore out of reset */
    if (iodelay_rst) {
        libwr2rf_writeh(0, iodelay_addr);
    }
            
    /* Assert reset for > 50 ns, timing here is ~ 0.5 us, 35 cycles x 16 ns  */
    libwr2rf_writeh(WR2RF_INIT_REGS_WRCORE_CTRL_RESET_N, iodelay_addr);
    libwr2rf_writeh(0,                                   iodelay_addr);
}




void
libwr2rf_init (void)
{
    /* 0. the OCXO */
    init_ocxo();

  /* 1. the PLL (ltc6950).  */
  /* 1.1 the SPI control for the PLL.  */
  libwr2rf_pll_spi_init();
  /* 1.2 the PLL itself.  */
  init_ltc6950 ();

  /* 2. FPGA clock.  */
  libwr2rf_clock_select_dds ();

#if 0

  /* 3. DDS (ad9910).  */
  libwr2rf_dds_spi_init();
  libwr2rf_dds_init();

  /* 4. RF DAC.  */
  for (unsigned i = 0; i < 2; i++)
    {
      /* 4.1 Init registers.  */
      libwr2rf_dac_init(i);

      /* 4.2 Set timing for inputs.  */
      libwr2rf_dac_port_timing(i);
    }
#endif

  /* 5. WR-Core */
  init_wrcore();

  /* 6. IOdelays */
  init_iodelay();
}
