#define _XOPEN_SOURCE
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <poll.h>
#include <termios.h>
#include <errno.h>
#include <arpa/inet.h>
#include "host.h"
#include "board.h"
#include "wr2rf_vme_regs.h"
#include "wr2rf_init_regs.h"
#include "wb_uart.h"
#include "regs.h"

/* [0x10]: REG Hardware Info Register */
#define SYSC_REG_HWIR 0x00000010

static uint32_t
libwr2rf_wrc_readl(unsigned off)
{
        return libwr2rf_16x32_readl(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRPC + off);
}

static void
libwr2rf_wrc_writel(unsigned off, uint32_t val)
{
        libwr2rf_16x32_writel(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRPC + off, val);
}

static uint32_t
libwr2rf_framerxtx_readl(unsigned off)
{
  return libwr2rf_16x32_readl (WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_FRAMERXTX + off);
}

static void
libwr2rf_framerxtx_writel(unsigned off, uint32_t val)
{
  //printf("writel %08x at %04x\n", val, off);
  libwr2rf_16x32_writel(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_FRAMERXTX + off, val);
}

static void
framerxtx_show (int argc, char **argv)
{
  unsigned prev = ~0;
  unsigned count;

  while (1)
    {
      count = libwr2rf_readh
        (WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_RXFRAME_COUNTER);
      if (count != prev) {
        printf ("RX frame count: %04x, ctrl: %04x, ftw: %04x%04x%04x\n",
                count,
                libwr2rf_readh(WR2RF_VME_REGS_INIT
                               + WR2RF_INIT_REGS_RXFRAME_CTRL),
                libwr2rf_readh(WR2RF_VME_REGS_INIT
                               + WR2RF_INIT_REGS_RXFRAME_FTW + 2),
                libwr2rf_readh(WR2RF_VME_REGS_INIT
                               + WR2RF_INIT_REGS_RXFRAME_FTW + 4),
                libwr2rf_readh(WR2RF_VME_REGS_INIT
                               + WR2RF_INIT_REGS_RXFRAME_FTW + 6));
        prev = count;
      }
    }
}

static void
framerxtx_last (int argc, char **argv)
{
  unsigned count;

  printf ("NCO forced: %x\n",
          !!(libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_LOC_OR_WRS)
             & WR2RF_INIT_REGS_NCO_LOC_OR_WRS_PARAMS_SEL));

  count = libwr2rf_readh
    (WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_RXFRAME_COUNTER);

  printf ("RX frame count: %04x, ctrl: %04x, ftw: %04x%04x%04x\n",
          count,
          libwr2rf_readh(WR2RF_VME_REGS_INIT
                         + WR2RF_INIT_REGS_RXFRAME_CTRL),
          libwr2rf_readh(WR2RF_VME_REGS_INIT
                         + WR2RF_INIT_REGS_RXFRAME_FTW + 2),
          libwr2rf_readh(WR2RF_VME_REGS_INIT
                         + WR2RF_INIT_REGS_RXFRAME_FTW + 4),
          libwr2rf_readh(WR2RF_VME_REGS_INIT
                         + WR2RF_INIT_REGS_RXFRAME_FTW + 6));
  printf ("RX frame counter: %04x\n",
          libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_RXFRAME_COUNTER));
}

static void
framerxtx_framerxtx (int argc, char **argv)
{
  printf ("RFFRame version:   %08x\n", libwr2rf_framerxtx_readl (0x00));
  printf ("Status:            %08x\n", libwr2rf_framerxtx_readl (0x04));
  printf ("TX_Period:         %08x\n", libwr2rf_framerxtx_readl (0x08));
  printf ("rx_out_data_time:  %08x\n", libwr2rf_framerxtx_readl (0x0c));
  printf ("simrf_ctl:         %08x\n", libwr2rf_framerxtx_readl (0x10));
  printf ("rfsim_scyc_len:    %08x\n", libwr2rf_framerxtx_readl (0x14));
  printf ("rfsim_scyc_inc:    %08x\n", libwr2rf_framerxtx_readl (0x18));
  printf ("rfsim_scyc_ctrl:   %08x\n", libwr2rf_framerxtx_readl (0x1c));
  printf ("dbg_rx_rfm_or_rfs: %08x\n", libwr2rf_framerxtx_readl (0x20));
  printf ("dbg_tx_rfm_or_rfs: %08x\n", libwr2rf_framerxtx_readl (0x24));
  printf ("ftw_sim_off msb:   %08x\n", libwr2rf_framerxtx_readl (0x28));
  printf ("ftw_sim_off lsb:   %08x\n", libwr2rf_framerxtx_readl (0x2c));
}

static void
framerxtx_send1 (int argc, char **argv)
{
  /* SCR: tx dbg=1 (counter), ftype=1 (RFm).  */
  libwr2rf_framerxtx_writel (0x04, (1 << 3) | (1 << 1) | (1 << 0));
}

static void
framerxtx_send (int argc, char **argv)
{
  /* TX period: every second  */
  libwr2rf_framerxtx_writel (0x08, 62500000);

  /* SCR: tx dbg=1 (counter), ftype=1 (RFm).  */
  libwr2rf_framerxtx_writel (0x04, (1 << 9) | (1 << 3) | (1 << 1));
}

static void
framerxtx_simul (int argc, char **argv)
{
  /* One frame every 20us (50_000 frame/sec)
     Pattern:
     * sync on PPS,
     * 600ms flat bottom (Frf=198.51 Mhz, ftw=0x16_8701_2c85
        len=30000
     * ramp-up to Frf=200.392Mhz
        len=37640, inc=0x5f32
     * flat-top: Frf=200.392Mhz, ftw=0x16_bdad_eee1
        len~=32460, (last 10 with reset)
     */
  /* FTW_SIM_OFFSET: low.  */
  libwr2rf_framerxtx_writel (0x28, 0x16);
  libwr2rf_framerxtx_writel (0x2c, 0x87012c85);

  /* SIMRF CTRL: CLEN=0, SCID=0  */
  libwr2rf_framerxtx_writel (0x10, 0x00);

  /* Set sub cycle length and increment.  */
  libwr2rf_framerxtx_writel (0x14, 30000);
  libwr2rf_framerxtx_writel (0x18, 0);
  libwr2rf_framerxtx_writel (0x1c, 0);

  /* SIMRF CTRL: CLEN=0, SCID=1  */
  libwr2rf_framerxtx_writel (0x10, 0x10);

  /* Set sub cycle length and increment.  */
  libwr2rf_framerxtx_writel (0x14, 37640);
  libwr2rf_framerxtx_writel (0x18, 0x5f32);
  libwr2rf_framerxtx_writel (0x1c, 0);

  /* SIMRF CTRL: CLEN=0, SCID=2  */
  libwr2rf_framerxtx_writel (0x10, 0x20);

  /* Set sub cycle length and increment.  */
  libwr2rf_framerxtx_writel (0x14, 10000 /*32450*/);
  libwr2rf_framerxtx_writel (0x18, 0);
  libwr2rf_framerxtx_writel (0x1c, 0);

  /* SIMRF CTRL: CLEN=0, SCID=3  */
  libwr2rf_framerxtx_writel (0x10, 0x30);

  /* Set sub cycle length and increment.  */
  libwr2rf_framerxtx_writel (0x14, 8);
  libwr2rf_framerxtx_writel (0x18, 0);
  libwr2rf_framerxtx_writel (0x1c, 1);

  /* SIMRF CTRL: CLEN=4, SCID=0, CPPSSTART  */
  libwr2rf_framerxtx_writel (0x10, (1 << 12) | 0x04);

  /* TX period: every 20us */
  libwr2rf_framerxtx_writel (0x08, 62500000 / 50000);

  /* SCR: tx dbg=2 (simul), ftype=1 (RFm).  */
  libwr2rf_framerxtx_writel (0x04, (1 << 9) | (1 << 3) | (2 << 1));
}

static void
nco_source (int argc, char **argv)
{
  unsigned ctrl;

  ctrl = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_LOC_OR_WRS);

  if (argc == 1
      || strcmp (argv[1], "status") == 0)
    {
      printf ("CTRL: %04x, NCO local(=1) network(=0): %x\n",
              ctrl,
              !!(ctrl & WR2RF_INIT_REGS_NCO_LOC_OR_WRS_PARAMS_SEL));
    }
  else if (strcmp (argv[1], "network") == 0)
    {
      ctrl &= ~WR2RF_INIT_REGS_NCO_LOC_OR_WRS_PARAMS_SEL;
      libwr2rf_writeh(ctrl, WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_LOC_OR_WRS);
    }
  else if (strcmp (argv[1], "local") == 0)
    {
      ctrl |= WR2RF_INIT_REGS_NCO_LOC_OR_WRS_PARAMS_SEL;
      libwr2rf_writeh(ctrl, WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_LOC_OR_WRS);
    }
  else
    printf ("unknown source sub-command %s\n", argv[1]);
}

static void
nco_reset (int argc, char **argv)
{
  unsigned ctrl;
  unsigned rst;

  ctrl = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_CTRL);
  rst  = !!(ctrl & WR2RF_INIT_REGS_NCO_CTRL_RESET_NCO);

  printf("CTRL: %04x\n", ctrl);

  if (! rst)
    ctrl = ctrl | WR2RF_INIT_REGS_NCO_CTRL_RESET_NCO;
  else
    ctrl = ctrl & ~WR2RF_INIT_REGS_NCO_CTRL_RESET_NCO;

  libwr2rf_writeh(ctrl, WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_CTRL);
}

static void
nco_wrctrl (int argc, char **argv)
{
  unsigned long  ctrl;
  unsigned short before;
  unsigned short after;

  if (argc != 2) {
      printf("Expected %s <val>\n", argv[0]);
      return;
  } else {
      ctrl = strtoul(argv[1], NULL, 0);

      before = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_CTRL);
      libwr2rf_writeh(ctrl, WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_CTRL);
      libwr2rf_writeh(WR2RF_INIT_REGS_NCO_UPDATE_VALID,
                      WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_UPDATE);
      after  = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_CTRL);
      printf("before=%04x after=%04x expected=%04lx %s\n",
             before, after, ctrl, (ctrl != after ? "ERROR" : ""));
  }
}

static void
nco_lcfg (int argc, char **argv)
{
    unsigned long long ftw = 0xD558BCE08;
    unsigned nco_ftw_addr    = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_H1_FTW;
//unsigned nco_reset_addr  = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_CTRL;
//    unsigned nco_update_addr = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_NCO_UPDATE;



    if (argc == 2) {
        ftw = strtoull(argv[1], NULL, 0);
    }
    else if (argc != 1) {
        printf("Unexpected arguments\n");
        return;
    }
    ftw = 0xfedcba9876543210;

    printf("Supplied ftw=0x%016llx\n", ftw);
    libwr2rf_writeh(ftw,     nco_ftw_addr+0);
    libwr2rf_writeh(ftw>>16, nco_ftw_addr+2);
    libwr2rf_writeh(ftw>>32, nco_ftw_addr+4);
    libwr2rf_writeh(ftw>>48, nco_ftw_addr+6);

    printf("done\n");
    printf("fpga_ftw=%04x.%04x.%04x.%04x\n",
           libwr2rf_readh(nco_ftw_addr+0),
           libwr2rf_readh(nco_ftw_addr+2),
           libwr2rf_readh(nco_ftw_addr+4),
           libwr2rf_readh(nco_ftw_addr+6));

    printf("Supplied ftw=0x%016llx\n", ftw);
    libwr2rf_writeh(ftw,     nco_ftw_addr+6);
    libwr2rf_writeh(ftw>>16, nco_ftw_addr+4);
    libwr2rf_writeh(ftw>>32, nco_ftw_addr+2);

    printf("fpga_ftw=%04x.%04x.%04x.%04x\n",
           libwr2rf_readh(nco_ftw_addr+0),
           libwr2rf_readh(nco_ftw_addr+2),
           libwr2rf_readh(nco_ftw_addr+4),
           libwr2rf_readh(nco_ftw_addr+6));


}

static void
svmup_pll_sel (int argc, char **argv)
{
  unsigned short after;
  unsigned short before;
  unsigned short pll;

  before = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_SVEC_MUP_CTRL);
  after  = before;
  pll    = !!(before & WR2RF_INIT_REGS_SVEC_MUP_CTRL_BCLK_RFCLK_SEL);

  if (! pll)
      after = after | WR2RF_INIT_REGS_SVEC_MUP_CTRL_BCLK_RFCLK_SEL;
  else
      after = after & ~WR2RF_INIT_REGS_SVEC_MUP_CTRL_BCLK_RFCLK_SEL;

  libwr2rf_writeh(after, WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_SVEC_MUP_CTRL);
  after = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_SVEC_MUP_CTRL);
  printf("svmup_pll_sel: before=%04x after=%04x\n", before, after);
}

static void
read_init_reg (int argc, char **argv)
{
  unsigned long addr;
  unsigned long data;

  if (argc != 2) {
      printf("Expected %s <val>\n", argv[0]);
      exit(EXIT_FAILURE);
  } else {
      addr = strtoul(argv[1], NULL, 0);
      data = libwr2rf_readh(WR2RF_VME_REGS_INIT + addr);

      printf("read_init_reg: addr: %04x, data: %04x \n", (unsigned)addr, (unsigned)data);
  }
}

static void
write_init_reg (int argc, char **argv)
{
  unsigned long addr;
  unsigned long data;

  if (argc != 3) {
      printf("Expected %s <val>\n", argv[0]);
      exit(EXIT_FAILURE);
  } else {
      addr = strtoul(argv[1], NULL, 0);
      data = strtoul(argv[2], NULL, 0);

      libwr2rf_writeh(data, WR2RF_VME_REGS_INIT + addr);
  }
}

static void
wrc_print_status (unsigned status)
{
  printf ("wrc status: %02x, link: %u, time: %u, ",
          status,
          (status & WR2RF_INIT_REGS_WRC_STATUS_LINKUP) ? 1 : 0,
          (status & WR2RF_INIT_REGS_WRC_STATUS_TIME_VALID) ? 1 : 0);
}

static void
wrc_print_time (void)
{
  unsigned tai[3];
  unsigned cyc[2];

  do
    {
      tai[2] = libwr2rf_readh(WR2RF_VME_REGS_INIT
                              + WR2RF_INIT_REGS_WRC_TAI + 6);
      tai[1] = libwr2rf_readh(WR2RF_VME_REGS_INIT
                              + WR2RF_INIT_REGS_WRC_TAI + 4);
      tai[0] = libwr2rf_readh(WR2RF_VME_REGS_INIT
                              + WR2RF_INIT_REGS_WRC_TAI + 2);
    } while (tai[2] != libwr2rf_readh(WR2RF_VME_REGS_INIT
                                          + WR2RF_INIT_REGS_WRC_TAI + 6));
  do
    {
      cyc[0] = libwr2rf_readh(WR2RF_VME_REGS_INIT
                              + WR2RF_INIT_REGS_WRC_CYCLES + 0);
      cyc[1] = libwr2rf_readh(WR2RF_VME_REGS_INIT
                              + WR2RF_INIT_REGS_WRC_CYCLES + 2);
    }
  while (cyc[0] != libwr2rf_readh(WR2RF_VME_REGS_INIT
                                  + WR2RF_INIT_REGS_WRC_CYCLES));
  printf ("tai: %04x%04x%04x, cycles: %04x%04x",
          tai[0], tai[1], tai[2],
          cyc[0], cyc[1]);
}

static void
wrc_time (int argc, char **argv)
{
  unsigned status;

  status = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRC_STATUS);
  wrc_print_status (status);
  if (1 || (status & WR2RF_INIT_REGS_WRC_STATUS_TIME_VALID))
    wrc_print_time();
  else
    printf ("(invalid)");
  putchar('\n');
}

static void
wrc_wait (int argc, char **argv)
{
  unsigned status;

  while (1)
    {
      status = libwr2rf_readh(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRC_STATUS);
      wrc_print_status (status);
      wrc_print_time();
      putchar('\n');
      if (status & WR2RF_INIT_REGS_WRC_STATUS_TIME_VALID)
        break;
      sleep(1);
    }
}

static void
wrc_cycles (int argc, char **argv)
{
  unsigned cyc[2];

  while (1)
    {
      cyc[0] = libwr2rf_readh(WR2RF_VME_REGS_INIT
                              + WR2RF_INIT_REGS_WRC_CYCLES + 0);
      cyc[1] = libwr2rf_readh(WR2RF_VME_REGS_INIT
                              + WR2RF_INIT_REGS_WRC_CYCLES + 2);
      printf ("cycles: %08x\n", (cyc[0] << 16) | cyc[1]);
  }
}

static uint32_t vuart_readl(int reg)
{
	return libwr2rf_wrc_readl(0x500 + reg);
}


static void vuart_writel(uint32_t value, int reg)
{
        libwr2rf_wrc_writel(0x500 + reg, value);
}

static int8_t wr_vuart_rx(void)
{
	int rdr = vuart_readl(UART_REG_HOST_RDR );
	return (rdr & UART_HOST_RDR_RDY) ? UART_HOST_RDR_DATA_R(rdr) : -1;
}

/**
 * It transmits a single byte
 * @param[in] vuart token from dev_map()
 */
static void wr_vuart_tx(char data)
{
        do {
                int sr = vuart_readl(UART_REG_SR );
                if (sr == -1)
                        exit(1);
                /* If the rx byte is not present (and therefore cannot be
                   read), it could be written.  */
                if (!(sr & UART_SR_RX_RDY))
                        break;
        } while (1);

	vuart_writel(UART_HOST_TDR_DATA_W(data), UART_REG_HOST_TDR );
}

static void
vuart (int argc, char **argv)
{
	struct termios old_tios, new_tios;
        struct pollfd pfd;
	int ret;
	char rx, tx;
        unsigned res;


        //libwr2rf_writel(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRC_PAGE0, 0x40004);
        //libwr2rf_writel(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRC_PAGE1, 0x10000);

        libwr2rf_be_writel(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRC_PAGE0, 0x40000);

        res = libwr2rf_wrc_readl (0x400 + SYSC_REG_HWIR);
        printf ("HWIR: %08x\n", res);

        fprintf(stderr, "[press C-a to exit]\n");

        tcgetattr(0, &old_tios);
        memcpy (&new_tios, &old_tios, sizeof (struct termios));

        new_tios.c_iflag &= ~(IGNBRK | BRKINT | PARMRK | ISTRIP
                              | INLCR | IGNCR | ICRNL | IXON);
        new_tios.c_oflag &= ~OPOST;
        new_tios.c_lflag &= ~(ECHO | ECHONL | ICANON | ISIG | IEXTEN);
        new_tios.c_cflag &= ~(CSIZE | PARENB);
        new_tios.c_cflag |= CS8 | ISIG;
        tcflush(0, TCIFLUSH);
        tcsetattr(0, TCSANOW, &new_tios);

        pfd.fd = 0;
        pfd.events = POLLIN;

	while(1) {
                ret = poll (&pfd, 1, 10);
                if (ret == 0)
                        ;
                else if (ret > 0 && (pfd.revents & POLLIN)) {
                        /* The user wrote something */
                        ret = read(0, &tx, 1);
                        if (ret != 1) {
                                fprintf(stderr, "nothing to read. Port disconnected?\n");
                                break;
                        }
                        /* If the user character is C-a, then kill */
                        if(tx == 0x01)
                                break;

                        wr_vuart_tx(tx);
                }
                else
                        break;

		/* Print all the incoming charactes */
                while (1) {
                        rx = wr_vuart_rx();
                        if (rx == -1)
                                break;
                        write(1, &rx, 1);
		}
	}

        tcsetattr(0, TCSANOW, &old_tios);
}

static void
fw_dump (int argc, char **argv)
{
  unsigned addr;
  unsigned len = 0x2000;
  unsigned j;

  if (argc > 1)
    len = strtoul (argv[1], NULL, 0);

  printf ("WRC memory dump of 0x%06x words:\n", len);

  for (addr = 0; addr < len; addr += 16) {
    if ((addr & 0xfff) == 0)
      libwr2rf_be_writel(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRC_PAGE1,
                         addr & (~0xfff));
    printf ("%06x:", addr);
    for (j = 0; j < 16; j += 4) {
        unsigned v = libwr2rf_wrc_readl(0x1000 + ((addr+j) & 0xfff));
      printf (" %08x", v);
    }
    printf("\n");
  }
}


static unsigned *
load_binary_file(const char *filename, long *size)
{
  int i;
  long sz;
  unsigned *buf;
  FILE *f;

  f = fopen(filename, "r");
  if (!f) {
    fprintf(stderr, "%s: %s\n", filename, strerror(errno));
    return NULL;
  }

  if (fseek(f, 0, SEEK_END) != 0) {
    fprintf(stderr, "%s: %s\n", filename, strerror(errno));
    fclose(f);
    return NULL;
  }

  sz = ftell(f);
  *size = sz;

  fseek(f, 0, SEEK_SET);

  buf = malloc((sz + 3) & ~3);
  if (!buf) {
    fprintf(stderr, "loading %s: %s\n", filename, strerror(errno));
    fclose(f);
    return NULL;
  }

  i = fread(buf, sz, 1, f);
  fclose(f);
  if (i < 0) {
    fprintf(stderr, "reading %s: %s\n", filename, strerror(errno));
    free(buf);
    return NULL;
  }
  if (i != 1) {
    fprintf(stderr, "%s: short read\n", filename);
    free(buf);
    return NULL;
  }

  return buf;
}

static void
fw_load (int argc, char **argv)
{
  unsigned *buf;
  long sz;

  if (argc != 2) {
    printf ("fw-load: need filename\n");
    return;
  }

  buf = load_binary_file(argv[1], &sz);
  if (buf == NULL)
    return;

  /* Reset LM32.  */
  libwr2rf_be_writel(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRC_PAGE0, 0x40000);
  libwr2rf_wrc_writel(0x400, 0x1deadbee);
  while (!(libwr2rf_wrc_readl(0x400) & 0x10000000))
    ;

  /* Load.  */
  for (unsigned i = 0; i < sz; i += 4) {
      unsigned v =       htonl(buf[i >> 2]);

    if ((i & 0xfff) == 0)
      libwr2rf_be_writel(WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_WRC_PAGE1,
                         i & (~0xfff));
    libwr2rf_wrc_writel(0x1000 + (i & 0xfff), v);//buf[i >> 2]);
  }

  /* Unreset.  */
  libwr2rf_wrc_writel(0x400, 0x0deadbee);
}

static void
pll_dump (int argc, char **argv)
{
  /* Simple test.  */
  libwr2rf_pll_spi_init();

  /* TODO:
     * check REV/PART (Reg 16)
     * check NO_REF = 0, NO_VCO = 0 (presence of clocks)
     * configure status
   */

  /* Note: Reg 16 is REV/PART.  */
  for (unsigned i = 0; i <= 0x16; i++)
    printf ("pll regs %02x: %02x\n", i, libwr2rf_pll_read(i));
}

static void
mmcm_status (int argc, char **argv)
{
  unsigned addr = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_CLOCK_STATUS;
  unsigned long val = libwr2rf_readh(addr);

  printf ("clock status: 0x%04lx\n", val);
  printf ("mmcm locked: 0x%04lx\n",
          val & WR2RF_INIT_REGS_CLOCK_STATUS_MMCM_LOCKED);
  printf ("shift busy: 0x%04lx\n",
          val & WR2RF_INIT_REGS_CLOCK_STATUS_SHIFT_BUSY);
}

static void
pll_status (int argc, char **argv)
{
  unsigned addr = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_PLL_STATUS;
  unsigned long val = libwr2rf_readh(addr);

  printf ("pll status: 0x%04lx\n", val);
  printf ("error:  0x%04lx\n",
          val & WR2RF_INIT_REGS_PLL_STATUS_ERROR);
  printf ("locked: 0x%04lx\n",
          val & WR2RF_INIT_REGS_PLL_STATUS_LOCKED);
}

static void
pll_init (int argc, char **argv)
{
  /* Simple test.  */
  libwr2rf_pll_spi_init();
  libwr2rf_init();
}

static void
dds_reset (int argc, char **argv)
{
  unsigned addr = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_DDS_CTRL;
  unsigned val;

  if (argc == 1)
    {
      val = libwr2rf_readh(addr);
      printf ("dds ctrl: 0x%04x\n", val);
    }
  else if (argc == 2)
    {
      if (strcmp (argv[1], "0") == 0)
        val = 0;
      else if (strcmp (argv[1], "1") == 0)
        val = WR2RF_INIT_REGS_DDS_CTRL_RESET;
      else {
        printf ("incorrect reset value (got %s)\n", argv[1]);
        return;
      }

      libwr2rf_writeh(val, addr);
    }
  else
    printf ("usage: %s [0|1]\n", argv[0]);
}

static void
dds_init (int argc, char **argv)
{
    unsigned dds_reset_addr = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_DDS_CTRL;

    libwr2rf_writeh(WR2RF_INIT_REGS_DDS_CTRL_RESET, dds_reset_addr);
    libwr2rf_writeh(0, dds_reset_addr);
  libwr2rf_dds_init();
}

struct dds_regs {
  const char *name;
  unsigned nbits;
  unsigned addr;
};

static const struct dds_regs dds_regs[] =
  {
   { "cfr1", 32, 0 },
   { "cfr2", 32, 1 },
   { "cfr3", 32, 2 },
   { "dac ", 32, 3 },
   { "iour", 32, 4 },
   { "ftw ", 32, 7 },
   { "pow ", 16, 8 },
   { "asf ", 32, 9 },
   { "sync", 32, 0xa },
   { "rmpl", 64, 0xb },
   { "rmps", 64, 0xc },
   { "rmpr", 32, 0xd },
   { "prf0", 64, 0xe },
   { "prf1", 64, 0xf },
   { "prf2", 64, 0x10 },
   { "prf3", 64, 0x11 },
   { "prf4", 64, 0x12 },
   { "prf5", 64, 0x13 },
   { "prf6", 64, 0x14 },
   { "prf7", 64, 0x15 },
   { "ram ", 32, 0x16 },
   { NULL, 0, 0 }
  };


static void
dds_dump (int argc, char **argv)
{
  for (const struct dds_regs *r = dds_regs; r->name; r++) {
    uint64_t v = libwr2rf_dds_read(r->addr, r->nbits);
    printf ("%s [@%02x] = %llx (%d bits)\n",
            r->name, r->addr, (unsigned long long)v, r->nbits);
  }
}

static void
set_rfout (int argc, char **argv)
{
  unsigned rf;
  unsigned addr;
  unsigned val;

  if (argc != 3) {
    printf ("usage: set-rfout 1|2 mix|dds|gnd\n");
    return;
  }

  if (strcmp(argv[1], "1") == 0)
    rf = WR2RF_INIT_RF_REGS_RF1;
  else if (strcmp(argv[1], "2") == 0)
    rf = WR2RF_INIT_RF_REGS_RF2;
  else {
    printf ("bad rf channel (got %s)\n", argv[1]);
    return;
  }

  if (strcmp(argv[2], "mix") == 0)
    val = WR2RF_INIT_RF_CH_REGS_COMMON_MIXER_EN | 0x6;
  else if (strcmp(argv[2], "dds") == 0)
    val = 0x4;
  else if (strcmp(argv[2], "gnd") == 0)
    val = 0x2;
  else {
    printf ("bad rfout mode (got %s)\n", argv[2]);
    return;
  }

  addr = WR2RF_VME_REGS_INIT
    + WR2RF_INIT_REGS_RF + rf + WR2RF_INIT_RF_CH_REGS_COMMON;
  libwr2rf_writeh(val, addr);
}

static void
ext_ref (int argc, char **argv)
{
  unsigned addr = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_PIN_CTRL;
  unsigned val;

  if (argc == 1) {
    val = libwr2rf_readh (addr);
    printf ("ext-ref dir: %04x (0:in, 1:out)\n", val);
    return;
  }
  if (argc == 2) {
    if (strcmp (argv[1], "in") == 0)
      val = 0;
    else if (strcmp (argv[1], "out") == 0)
      val = WR2RF_INIT_REGS_PIN_CTRL_EXT_REF_DIR;
    else {
      printf ("ext-ref: incorrect direction.  expect in/out, got %s\n",
              argv[1]);
      return;
    }
    libwr2rf_writeh (val, addr);
    return;
  }

  printf ("usage: ext-ref [in|out]\n");
}

static void
hwver (int argc, char **argv)
{
  unsigned addr = WR2RF_VME_REGS_INIT + WR2RF_INIT_REGS_HWVER;

  printf ("hw version: 0x%04x\n", libwr2rf_readh(addr));
}

static void
dac_init (int argc, char **argv)
{
  unsigned ch;
  if (argc != 2) {
    printf ("usage: dac-init 1|2\n");
    return;
  }

  if (strcmp(argv[1], "1") == 0)
    ch = DAC1_CS;
  else if (strcmp(argv[1], "2") == 0)
    ch = DAC2_CS;
  else {
    printf ("bad rf channel (got %s)\n", argv[1]);
    return;
  }

  libwr2rf_dac_init (ch);
}


static void
dac_timing (int argc, char **argv)
{
  unsigned ch;

  if (argc != 2) {
    printf ("usage: dac-init 1|2\n");
    return;
  }

  if (strcmp(argv[1], "1") == 0)
    ch = DAC1_CS;
  else if (strcmp(argv[1], "2") == 0)
    ch = DAC2_CS;
  else {
    printf ("bad rf channel (got %s)\n", argv[1]);
    return;
  }

  libwr2rf_dac_port_timing(ch);
}

struct dac_reg {
  const char *name;
  unsigned addr;
};

static const struct dac_reg dac_regs[] =
  {
   { "spi_control ", DAC_REG_SPI_CONTROL  },
   { "data_control", DAC_REG_DATA_CONTROL },
   { "power_down  ", DAC_REG_POWER_DOWN   },
   { "setup_hold  ", DAC_REG_SETUP_HOLD   },
   { "timing_adj  ", DAC_REG_TIMING_ADJ   },
   { "seek        ", DAC_REG_SEEK         },
   { "mix_mode    ", DAC_REG_MIX_MODE     },
   { "dac1_fsc_hi ", DAC_REG_DAC1_FSC_HI  },
   { "dac1_fsc_lo ", DAC_REG_DAC1_FSC_LO  },
   { "auxdac1_hi  ", DAC_REG_AUXDAC1_HI   },
   { "auxdac1_lo  ", DAC_REG_AUXDAC1_LO   },
   { "dac2_fsc_hi ", DAC_REG_DAC2_FSC_HI  },
   { "dac2_fsc_lo ", DAC_REG_DAC2_FSC_LO  },
   { "auxdac2_hi  ", DAC_REG_AUXDAC2_HI   },
   { "auxdac2_lo  ", DAC_REG_AUXDAC2_LO   },
   { "version     ", DAC_REG_VERSION      },
   { NULL, 0}
  };

static void
dac_dump (int argc, char **argv)
{
  unsigned ch;

  if (argc != 2) {
    printf ("usage: dac-dump 1|2\n");
    return;
  }

  if (strcmp(argv[1], "1") == 0)
    ch = DAC1_CS;
  else if (strcmp(argv[1], "2") == 0)
    ch = DAC2_CS;
  else {
    printf ("bad rf channel (got %s)\n", argv[1]);
    return;
  }

  for (const struct dac_reg *r = dac_regs; r->name; r++) {
    unsigned val = libwr2rf_dac_read(ch, r->addr);
    printf ("%s [@%02x] = 0x%08x\n", r->name, r->addr, val);
  }
}

static void cmd_help (int argc, char **argv);

struct cmds {
  const char *name;
  void (*cmd)(int argc, char **argv);
  const char *help;
};

static struct cmds cmds[] =
  {
   { "help", cmd_help, "show help" },
   { "hwver", hwver, "print hw version" },
   { "show", framerxtx_show, "show received ftw" },
   { "last", framerxtx_last, "display last ftw" },
   { "framerxtx", framerxtx_framerxtx, "display framerxtx registers" },
   { "send1", framerxtx_send1, "send one frame" },
   { "send", framerxtx_send, "send counter frames" },
   { "simul", framerxtx_simul, "send a ramp" },
   { "source", nco_source, "select nco source" },
   { "reset", nco_reset, "toggle nco reset" },
   { "nco_wrctrl", nco_wrctrl, "write nco ctrl register" },
   { "nco_lcfg", nco_lcfg, "configure the nco for local register access" },
   { "pllsel", svmup_pll_sel, "toggle pll_sel"},
   { "read_init_reg", read_init_reg, "write init ctrl register"},
   { "write_init_reg", write_init_reg, "write init ctrl register"},
   { "time", wrc_time, "display wrc time & status" },
   { "cycles", wrc_cycles, "display wrc cycles (continuously)" },
   { "wait", wrc_wait, "wait until wrc is sync-ed"},
   { "mmcm-status", mmcm_status, "display fpga mmcm status" },
   { "pll-status", pll_status, "display ltc6950 pll status" },
   { "pll-dump", pll_dump, "dump ltc6950 pll registers"},
   { "pll-init", pll_init, "start ltc6950 pll, switch to pll clock"},
   { "vuart", vuart, "connect to WRC vuart"},
   { "fw-dump", fw_dump, "dump WRC firmware"},
   { "fw-load", fw_load, "load WRC firmware"},
   { "dds-reset", dds_reset, "control ad9910 dds reset"},
   { "dds-init", dds_init, "init ad9910 dds"},
   { "dds-dump", dds_dump, "dump ad9910 dds registers"},
   { "dac-init", dac_init, "init ad9783 iq-dac"},
   { "dac-timing", dac_timing, "adjust data timing of ad9783 iq-dac"},
   { "dac-dump", dac_dump, "dump ad9783 iq-dac registers"},
   { "set-rfout", set_rfout, "set muxers for rfout"},
   { "ext-ref", ext_ref, "set or read ext_ref (pps+10Mhz) direction"},
   { NULL, NULL, NULL }
  };

static void
cmd_help (int argc, char **argv)
{
  const struct cmds *cmd;

  for (cmd = cmds; cmd->name; cmd++)
    printf ("%-10s  %s\n", cmd->name, cmd->help);
}

int
main (int argc, char **argv)
{
  int c;
  unsigned slot = 0;
  int i;

  while ((c = getopt (argc, argv, "s:h")) != -1)
    switch (c)
      {
      case 's':
	slot = strtol (optarg, NULL, 0);
	break;
      case 'h':
	printf ("usage: %s [-h] -s SLOT ACTION\n", argv[0]);
	return 1;
      case '?':
	fprintf (stderr, "incorrect option, try %s -h\n", argv[0]);
	return 2;
      }

  if (optind >= argc)
    {
      printf ("no action (try help)\n");
      return 1;
    }

  if (argc - optind == 1 && strcmp (argv[optind], "help") == 0)
    {
      cmd_help (argc - optind, argv + optind);
      return 0;
    }

  if (libwr2rf_host_init_by_slot (slot) != 0)
    return 1;

  for (i = 0; cmds[i].name; i++)
    if (strcmp (argv[optind], cmds[i].name) == 0)
      {
        cmds[i].cmd(argc - optind, argv + optind);
        return 0;
      }
  printf ("unknown action %s, try help\n", argv[optind]);
  return 1;
}
